/* 
   Copyright (C) 1992 Free Software Foundation, Inc.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

/*
   This program have been made by: Gregory VANDENBROUCK , vdb@pipo.com
   Please, send him/them a mail ( possibly with diff and/or correction )
   If you find any BUG, or if you do nay improvement */

#include "Dat.H"
#include "Debug.H"

Dat::Dat(const char ** CLES, 
	 unsigned int * Ze_tailles,
	 unsigned int * MAX_CH, 
	 unsigned int * MAX_L)
  :tailles(Ze_tailles),MAX(MAX_CH),MAXL(MAX_L),
   Modif_Chp(0),Modif_Num(0),Modif_List(0), _Cle(CLES)
{  init(); }
 
Dat::~Dat()
{ del(); }

void Dat::MetNull(void)
{ 
  unsigned int i;
  for (i=0;i<tailles[0];i++) Le_Champ[i].efface();
  for (i=0;i<tailles[1];i++) Le_Num[i]=0;
  for (i=0;i<tailles[2];i++) La_Liste[i].MetNull();
}

void Dat::init(void)
{
  unsigned int i;
  i = tailles[0];
  if ( i ) // On a des champs chaines de car 
    {
      Le_Champ  = new String [i];
      Modif_Chp = new int[i];
    }
  i=tailles[1];
  if ( i ) // on a des champs numeriks
    {
      Le_Num    = new unsigned long[i];
      Modif_Num = new int[i];
      for (unsigned int k=0; k< i; k++) Le_Num[k]=0;
    }
  i=tailles[2];
  if ( i ) // On a des listes 
    {
      La_Liste   = new LISTE[i];
      Modif_List = new int[i];
      for (unsigned int k=0; k< i; k++) 
	  La_Liste[k].NewMax(MAXL[k]);
    }
}

void Dat::del(void)
{
  if ( tailles[0] ) // On a des champs chaines de car !
    {
      delete [] Le_Champ;
      delete [] Modif_Chp;
    }
   if ( tailles[1] ) // on a des champs numeriks
     {
       delete [] Le_Num;
       delete [] Modif_Num;
     }
  if ( tailles[2] ) // On a des listes 
    {
      delete [] La_Liste;
      delete [] Modif_List;
    }
}

const String & Dat::St(unsigned int i)
{
  if ( i >= tailles[0] )
    {
      Debug(2,"Dat::St(): depacement de capacite\n");
      return Le_Champ[0];
    }
  return Le_Champ[i];
}

void Dat::SetSt(unsigned int i, const String &  chaine)
{
  if ( i >= tailles[0] )
    {
      Debug(2,"Dat::SetSt: depacement de capacite\n");
      return ;
    } 
  Le_Champ[i]=chaine;
  Modif_Chp[i]=1;
  if ( chaine.length() > (unsigned)MAX[i] )
    {
      //      Debug(0,"Dat::SetSt(): string [%s] truncated\n",chaine.data());
      Le_Champ[i].SetCar(MAX[i],0);
    }
}

unsigned long int Dat::Num(unsigned int i)
{
  if ( i >= tailles[1] )
    {
      Debug(2,"Dat::Num: depacement de capacite\n");
      return 0;
    }
  return Le_Num[i];
}

void Dat::Inc(unsigned int i,int add)
{
  if ( i >= tailles[1] )
    {
      Debug(2,"Dat::Num: depacement de capacite\n");
      return ;
    }
  Le_Num[i] += add;
  Modif_Num[i]=1;
}

int Dat::Set(unsigned int i, unsigned long flag)
{
  return ( Num(i) & flag );
}

void Dat::SetNum(unsigned int i, unsigned long val)
{
  if ( i >= tailles[1] )
    {
      Debug(2,"Dat::SetNum: depacement de capacite\n");
      return ;
    }
  Le_Num[i]=val;
  Modif_Num[i]=1;
}

void Dat::Swap(unsigned int i, unsigned long flag)
{
  if (  i >= tailles[1] )
    {
      Debug(2,"Dat::Swap: depacement de capacite\n");
      return ;
    }
  Le_Num[i]  ^= flag;
  Modif_Num[i]=1;
}

LISTE * Dat::L(unsigned int i)
{
  if ( i >= tailles[2] )
    {
      Debug(2,"Dat::L: depacement de capacite\n");
      return 0;
    }
  return La_Liste + i;
  Modif_List[i]=1;
}

unsigned int Dat::MaxCh(unsigned int i)
{
  if ( i >= tailles[0] )
    {
      Debug(2,"Dat::MaxCh: depacement de capacite\n");
      return 0;
    }
  else return MAX[i];
}

unsigned int Dat::NC(void)
{
  return tailles[0];
}

unsigned int Dat::NN(void)
{
  return tailles[1];
}

unsigned int Dat::NL(void)
{
  return tailles[2];
}

unsigned int Dat::total(void)
{
  return tailles[0]+tailles[1]+tailles[2];
}

const char *  Dat::Cle(unsigned int i)
{
  if ( i >= total() )
    {
      Debug(2,"Dat::Cle: depassement de capacite \n");
      return "";
    }
  return _Cle[i];
}
