/* ***************************************************************** *
 * G_Path                                                            *
 * ----------------------------------------------------------------- *
 * Version . . . . . . . . . . . . . . . . . . . . . . . . . . 1.2.0 *
 * ================================================================= *
 * Creation date . . . . . . . . . . . . . . . . . . ??? ?? ??? 1997 *
 * Modification date . . . . . . . . . . . . . . . . Fri 13 Nov 1998 *
 * ================================================================= *
 * Copyright (C) 1997-1998 Free Software Foundation, Inc.            *
 *                                                                   *
 * This program  is  free software;  you can  redistribute it and/or *
 * modify it  under the terms  of the GNU  General Public License as *
 * published by  the Free Software Foundation;  either version 2, or *
 * (at your option) any later version.                               *
 *                                                                   *
 * This program  is distributed in the hope  that it will be useful, *
 * but WITHOUT  ANY WARRANTY;  without even  the implied warranty of *
 * MERCHANTABILITY  or  FITNESS  FOR A  PARTICULAR PURPOSE.  See the *
 * GNU General Public License for more details.                      *
 *                                                                   *
 * You should have received a copy of the GNU General Public License *
 * along  with  this  program;  if not,  write to  the Free Software *
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.         *
 *                                                                   *
 * ================================================================= *
 * Authors                                                           *
 *   Gregory VANDENBROUCK <vdb@pipo.com>                             *
 *   Sebastien APERGHIS-TRAMONI <saper@esil.univ-mrs.fr>             *
 *                                                                   *
 * If you find any bug, or if you do any improvment, please send us  *
 * a mail (possibly with diff and/or correction).                    *
 *                                                                   *
 * ***************************************************************** */


/* ***************************************************************** *
 * standard includes                                                 *
 * ***************************************************************** */
#include <string.h>
#include <errno.h>
#include <signal.h>
#include <unistd.h>


/* ***************************************************************** *
 * Pipo-BBS includes                                                 *
 * ***************************************************************** */
#include "G_Path.H"
#include "CONFIG.H"



/* ***************************************************************** *
 * TIMEOUT()                                                         *
 * -------                                                           *
 * description                                                       *
 *   This is an internal function which is called when an SIGALRM is *
 *   raised, i.e. when the timeout has expired.                      *
 *                                                                   *
 * ***************************************************************** */
#ifdef CGI
static void TIMEOUT()
#else
static void TIMEOUT(int)
#endif
{
    fprintf(stderr, "\n-= TIMEOUT =-\nSorry, this application can't last so long \n");
    fflush(stdout);
    fflush(stderr);
    exit(1);
}


/* ***************************************************************** *
 * G_Path()                                                          *
 * ------                                                            *
 * description                                                       *
 *   This is the G_Path constructor. The timeout argument specify    *
 *   the maximum live of the object.                                 *
 *                                                                   *
 * ***************************************************************** */
G_Path::G_Path(int timeout) {
    void (*old_handler)(int);
    
    if(chdir(bin_home) == -1) { 
        perror("G_Path: init");
        printf("Unvalid path: '%s'\n", bin_home);
        throw;
    }
    
    // We install a custom SIGALRM handler to prevent an 
    // application from running for too long time on the server.
    
    old_handler = signal(SIGALRM, TIMEOUT);
    
    if(old_handler == SIG_ERR) {
        fprintf(stderr, "G_Path: init: can't assign signal handler SIGALRM.\n");
    }
    
    alarm(timeout);
}


/* ***************************************************************** *
 * SetTimeOut()                                                      *
 * ----------                                                        *
 * description                                                       *
 *   This function allows to set a new timeout.                      *
 *                                                                   *
 * ***************************************************************** */
int G_Path::SetTimeOut(int timeout) {
    return alarm(timeout);
}


/* ***************************************************************** *
 * GetBinHome()                                                      *
 * ----------                                                        *
 * description                                                       *
 *   This function returns the path of a file in the bin/ directory. *
 *                                                                   *
 * ***************************************************************** */
char * G_Path::GetBinHome(char * file) { 
    char *  path = new char[strlen(file) + strlen(bin_home) + 2];
    strcpy(path, bin_home);
    strcat(path, "/");
    strcat(path, file);
    return path; 
}


/* ***************************************************************** *
 * GetEtcHome()                                                      *
 * ----------                                                        *
 * description                                                       *
 *   This function returns the path of a file in the etc/ directory. *
 *                                                                   *
 * ***************************************************************** */
char * G_Path::GetEtcHome(char * file) {  
    char *  path = new char[strlen(file) + strlen(etc_home) + 2];
    strcpy(path, etc_home);
    strcat(path, "/");
    strcat(path, file);
    return path; 
} 


/* ***************************************************************** *
 * GetEtcHome()                                                      *
 * ----------                                                        *
 * description                                                       *
 *   This function returns the path of a file in the directory dir   *
 *   of the etc/ directory.                                          *
 *                                                                   *
 * ***************************************************************** */
char * G_Path::GetEtcHome(char * dir, char * file) {  
    char *  path = new char[strlen(file) + strlen(dir) + strlen(etc_home) + 3];
    strcpy(path, etc_home);
    strcat(path, "/");
    strcat(path, dir);
    strcat(path, "/");
    strcat(path, file);
    return path; 
} 


/* ***************************************************************** *
 * GetUsrHome()                                                      *
 * ----------                                                        *
 * description                                                       *
 *   This function returns the path of a file in the usr/ directory. *
 *                                                                   *
 * ***************************************************************** */
char * G_Path::GetUsrHome(char * file) {  
    char *  path = new char[strlen(file) + strlen(etc_home) + strlen(user_rep) + 3];
    strcpy(path, etc_home);
    strcat(path, "/");
    strcat(path, user_rep);
    strcat(path, "/");
    strcat(path, file);
    return path; 
} 


/* ***************************************************************** *
 * GetUserRep()                                                      *
 * ----------                                                        *
 * description                                                       *
 *   This function returns the path of a file in the users/          *
     directory.                                                      *
 *                                                                   *
 * ***************************************************************** */
char * G_Path::GetUserRep(char * file) {  
    char *  path = new char[strlen(file) + strlen(etc_home) + strlen(user_rep) + 3];
    strcpy(path, etc_home);
    strcat(path, "/");
    strcat(path, user_rep);
    strcat(path, "/");
    strcat(path, file);
    return path; 
}


/* ***************************************************************** *
 * GetServerApp()                                                    *
 * ------------                                                      *
 * description                                                       *
 *   This function returns the path to the server program.           *
 *                                                                   *
 * ***************************************************************** */
char * G_Path::GetServerApp(void) {
    return GetBinHome((char *)serveur);
}


/* ***************************************************************** *
 * GetClientApp()                                                    *
 * ------------                                                      *
 * description                                                       *
 *   This function returns the path to the client program.           *
 *                                                                   *
 * ***************************************************************** */
char * G_Path::GetClientApp(void) {
    return GetBinHome((char *)client);
}


/* ***************************************************************** *
 * GetProprio()                                                      *
 *                                                                   *
 * ***************************************************************** */
const char * G_Path::GetProprio(void) {
    return serveur;
}


/* ***************************************************************** *
 * GetUserFile()                                                     *
 * -----------                                                       *
 * description                                                       *
 *   This function returns the path of the given user file.          *
 *                                                                   *
 * ***************************************************************** */
char * G_Path::GetUserFile(const String & User, const char * File) {
    char *  path;
    char    dir[3];
    
    dir[0] = User[0];
    dir[1] = User[1];
    dir[2] = 0;
    
    path = new char[ strlen(etc_home) + strlen(user_rep) + strlen(dir) + 
                     strlen(User.data()) + strlen(File) + 5 ];
    
    strcpy(path, etc_home);
    strcat(path, "/");
    strcat(path, user_rep);
    strcat(path, "/");
    strcat(path, dir);
    strcat(path, "/");
    strcat(path, User.data());
    strcat(path, "/");
    strcat(path, File);
    
    return(path);
}


/* ***************************************************************** *
 * GetCommFile(void)                                                 *
 *                                                                   *
 * ***************************************************************** */
char * G_Path::GetCommFile(void) {
    char *  path = new char[strlen(data_home) + strlen(nom_pipe) + 2];
    strcpy(path, data_home);
    strcat(path, "/");
    strcat(path, nom_pipe);
    return path;
}


/* ***************************************************************** *
 * GetCommFile(int)                                                  *
 *                                                                   *
 * ***************************************************************** */
char * G_Path::GetCommFile(int i) {
    char *  path = new char[strlen(data_home) + 31];
    sprintf(path, "%s%d", data_home, i);
    return path;
}


/* ***************************************************************** *
 * GetCommFile(char *)                                               *
 *                                                                   *
 * ***************************************************************** */
char * G_Path::GetCommFile(const char * file) {
    char *  path = new char[strlen(data_home) + strlen(file) + 2];
    strcpy(path, data_home);
    strcat(path, "/");
    strcat(path, file);
    return path;
}


/* ***************************************************************** *
 * GetEmoteChar()                                                    *
 *                                                                   *
 * ***************************************************************** */
char * G_Path::GetEmote(char c) {
    char *  path = new char[strlen(etc_home) + strlen(emote) + 3];
    sprintf(path, "%s/%s%c", etc_home, emote, c);
    return path;
}


/* ***************************************************************** *
 * GetRoomFile()                                                     *
 *                                                                   *
 * ***************************************************************** */
char * G_Path::GetRoomFile(const char * file, int num) {
    char *  path = new char[strlen(etc_home) + strlen(room) + strlen(file) + 20];
    sprintf(path, "%s/%s/%03d/%s", etc_home, room, num, file);
    return path;
}


/* ***************************************************************** *
 * GetRoomInfo()                                                     *
 * -----------                                                       *
 * description                                                       *
 *   This function returns the path of the info file for the room    *
 *   number num, in the language lang.                               *
 *                                                                   *
 * ***************************************************************** */
char * G_Path::GetRoomInfo(unsigned short num, unsigned long lang) {
    char *  path = new char[strlen(etc_home) + strlen(room) + 20];
    unsigned long k = 0;
    
    for(long kk = 0; kk < NB_langues; kk++ ) {
        if((1 << kk) == (signed)lang) {
            k = kk; break;
        }
    }
    
    sprintf(path, "%s/%s/%03d/RI.%s", etc_home, room, num, Rep_Langues[k]);
    
    return path;
}


/* ***************************************************************** *
 * GetKHFile()                                                       *
 *                                                                   *
 * ***************************************************************** */
FILE * G_Path::GetKHFile(const char * ch, const char * type) {
    FILE *  file = NULL;
    char *  path = new char[strlen(etc_home) + strlen(KH) + strlen(ch) + 1];
    
    strcpy(path, etc_home);
    strcat(path, KH);
    strcat(path, ch);
    
    file = fopen(path, type);
    delete [] path;
    
    if(file == NULL) { 
        perror("G_Path::GetKHFile()");
        fprintf(stderr, "Can't open '%s' in %s mode.\n", ch, "append");
    }
    return file;
}

char * G_Path::GetLogFile(const char * ch)
{
  char *  path = new char[strlen(log_home) + strlen(ch) + 1];
  
  strcpy(path, log_home);
  strcat(path, ch);
  return path;
}

/* ***************************************************************** *
 * AppendLogFile()                                                   *
 *                                                                   *
 * ***************************************************************** */
FILE * G_Path::AppendLogFile(const char * ch) {
    FILE *  file = NULL;
    char *  path = GetLogFile(ch);
    
    file = fopen(path, "a");
    delete [] path;
    
    if(file == NULL) { 
        perror("G_Path::AppendLogFile()");
        fprintf(stderr, "Can't open '%s' in %s mode.\n", ch, "append");
    }
    return file;
}


/* ***************************************************************** *
 * Nblg()                                                            *
 * ----                                                              *
 * description                                                       *
 *   This function returns the total number of available languages.  *
 *                                                                   *
 * ***************************************************************** */
unsigned int G_Path::Nblg(void) {
    return NB_langues;
}


/* ***************************************************************** *
 * Lg()                                                              *
 * --                                                                *
 * description                                                       *
 *   This function returns the string corresponding to the language  *
 *   number i.                                                       *
 *                                                                   *
 * ***************************************************************** */
char * G_Path::Lg(int i) {
    return Rep_Langues[i];
}

