/* ***************************************************************** *
 * Login                                                             *
 * ----------------------------------------------------------------- *
 * Version . . . . . . . . . . . . . . . . . . . . . . . . . . 1.1.4 *
 * ================================================================= *
 * Creation date . . . . . . . . . . . . . . . . . . Tue 01 Sep 1998 *
 * Modification date . . . . . . . . . . . . . . . . Sun 07 Feb 1999 *
 * ================================================================= *
 * Copyright (C) 1997-1998 Free Software Foundation, Inc.            *
 *                                                                   *
 * This program  is  free software;  you can  redistribute it and/or *
 * modify it  under the terms  of the GNU  General Public License as *
 * published by  the Free Software Foundation;  either version 2, or *
 * (at your option) any later version.                               *
 *                                                                   *
 * This program  is distributed in the hope  that it will be useful, *
 * but WITHOUT  ANY WARRANTY;  without even  the implied warranty of *
 * MERCHANTABILITY  or  FITNESS  FOR A  PARTICULAR PURPOSE.  See the *
 * GNU General Public License for more details.                      *
 *                                                                   *
 * You should have received a copy of the GNU General Public License *
 * along  with  this  program;  if not,  write to  the Free Software *
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.         *
 *                                                                   *
 * ================================================================= *
 * Authors                                                           *
 *   Sebastien APERGHIS-TRAMONI <saper@esil.univ-mrs.fr>             *
 *   Gregory VANDENBROUCK <vdb@pipo.com>                             *
 *                                                                   *
 * If you find any bug, or if you do any improvment, please send us  *
 * a mail (possibly with diff and/or correction).                    *
 *                                                                   *
 * ***************************************************************** */


/* ***************************************************************** *
 * standard includes                                                 *
 * ***************************************************************** */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>
#include <time.h>
#include <fcntl.h>
#include <unistd.h>


/* ***************************************************************** *
 * Unix includes                                                     *
 * ***************************************************************** */
#include <sys/types.h>
#include <sys/stat.h>
#include <pwd.h>
#include <termios.h>
#include <termio.h>


/* ***************************************************************** *
 * Pipo-BBS includes                                                 *
 * ***************************************************************** */
#include "MAX.H"
#include "consts.H"
#include "G_Path.H"
#include "String.H"


/* ***************************************************************** *
 * Login includes                                                    *
 * ***************************************************************** */
#include "Login.h"
#include "utils.h"
#include "mail.h"


/* ***************************************************************** *
 * defines                                                           *
 * ***************************************************************** */
#define MAX_LEN         50
#define BUF_SIZE        1024

#define PASSWORD_CHAR   '*'


/* ***************************************************************** *
 * enums                                                             *
 * ***************************************************************** */
enum TermModes {
    TERM_MODE_RAW  = 1, 
    TERM_MODE_SANE = 2
};

enum MinLens {
    FIRST_NAME_MIN_LEN  = 3, 
    LAST_NAME_MIN_LEN   = 5, 
    EMAIL_MIN_LEN       = 10
};


/* ***************************************************************** *
 * prototypes                                                        *
 * ***************************************************************** */
int     main                (int argc, char ** argv);
void    terminate           (void);
void    CheckBbsRunning     (void);
int     CheckAuthorizedHost (int only_new);
char *  Language            (int language);
void    SetTerminalMode     (int mode);


/* ***************************************************************** *
 * global variables                                                  *
 * ***************************************************************** */
static char         program[]               = "Gnu's Pipo-BBS Login Program";
static char         version[]               = "1.1.4,1";

static char         Bbs_Down_file[]         = ".down";
static char         Bbs_NoNew_file[]        = ".new";
static char         Taboo_file[]            = "Hello/Taboo";
static char         HostDeny_file[]         = "Hello/HostDeny";
static char         Users_List_file[]       = "users/liste";
static char         User_Infos_file[]       = "infos";
static char         User_Passwd_file[]      = "pswd";

static char         hostname[MAX_LEN+1]     ;

static termios      Term                    ;
static tcflag_t     oldCtrl                 ;
static int          oldVMin                 ;


/* ***************************************************************** *
 * shared variables                                                  *
 * ***************************************************************** */
G_Path              G(LOGIN_MAX_IDLE)       ;



/* ***************************************************************** *
 * main()                                                            *
 *                                                                   *
 * ***************************************************************** */
int main(int argc, char ** argv) {
    Login  user;
    int    i = 0;
    
    if(argc <= 1) {
        //fprintf(stderr, "usage: Login <host>\n");
        strcpy(hostname,"<unknown host>");
        
    } else {
        strcpy(hostname, argv[1]);
    }
    
    CheckBbsRunning();
    CheckAuthorizedHost(false);
    
    print_file(G.GetEtcHome("msg/hello"));
    printf("This is GNU's Pipo-BBS version %s.\n", "1.2");
    
    SetTerminalMode(TERM_MODE_RAW);
    
    // Executing the successive menus.
    for(i=0; i<MAX_TRIES; i++) {
        
        switch(user.Logon()) {
            
            case 0: // Unvalid login
                break;
                 
            case 1: // Valid login
                user.Launch();
                break;
                 
            case 2: // New user
                user.NewUser();
                break;
        }
    }
    
    return 0;
}


/* ***************************************************************** *
 * terminate()                                                       *
 *                                                                   *
 * ***************************************************************** */
void terminate(void) {
    fprintf(stderr, "%s: terminate()\n", program);
    exit(1);
}


/* ***************************************************************** *
 * CheckBbsRunning()                                                 *
 * ---------------                                                   *
 * description                                                       *
 *   This function checks if the server is running.                  *
 *                                                                   *
 * ***************************************************************** */
void CheckBbsRunning(void) {
    FILE *  file = NULL;
    char    command[BUF_SIZE+1];
    int     status = 0;
    
    // First check: is the server running ?
    
    sprintf(command, "ps aux | grep %s | grep -v grep > /dev/null", G.GetProprio());
    status = system(command);
    
    if(status != 0) {
        printf("\033c\a\n"
               " .----------------------------------------------------------. \n"
               " | We are sorry,  but  Atlantis BBS  is currently  down for |.\n"
               " | unknown technical problems.                              ||\n"
               " | Please try to connect again in few minutes.              ||\n"
               " |                                                          ||\n"
               " | If this problem persists  for a long time,  please email ||\n"
               " | one of the administrators below.                         ||\n"
               " |                                                          ||\n"
               " |   vdb@pipo.com                                           ||\n"
               " |   saper@esil.univ-mrs.fr                   The Atlantis  ||\n"
               " |   nort@pipo.com                              BBS Team    ||\n"
               " `__________________________________________________________'|\n"
               "   ----------------------------------------------------------'\n"
        );
        sleep(SLEEP_TIME);
        exit(0);
    }
    
    // Second check: does the server accept connections ?
    
    file = fopen(G.GetBinHome(Bbs_Down_file), "r");
    
    if(file != NULL) {
        printf("\033c\a\n"
               " .----------------------------------------------------------. \n"
               " | We are sorry,  but  Atlantis BBS  is currently  down for |.\n"
               " | administration  reasons,  therefore  all connections are ||\n"
               " | refused for now.                                         ||\n"
               " |                                                          ||\n"
               " | Please try to connect again in few minutes.              ||\n"
               " `__________________________________________________________'|\n"
               "   ----------------------------------------------------------'\n"
        );
        fclose(file);
        sleep(SLEEP_TIME);
        exit(0);
    }
    
}


/* ***************************************************************** *
 * CheckAuthorizedHost()                                             *
 * -------------------                                               *
 * description                                                       *
 *   This function checks if the user's host is in the hosts deny    *
 *   list (i.e. the hosts that are unauthorized). The argument       *
 *   only_new indicates if we want to check global access to this    *
 *   host (only_new = false) or only prevent account creation        *
 *   (only_new = true).                                              *
 *                                                                   *
 * ***************************************************************** */
int CheckAuthorizedHost(int only_new) {
    FILE *  file = NULL;
    char *  path = NULL;
    char    buf[BUF_SIZE+1];
    char *  line = NULL;
    char *  host = NULL;
    char *  comments = NULL;
    
    path = G.GetEtcHome(HostDeny_file);
    
    file = fopen(path, "r");
    
    if(file == NULL) {
        fprintf(stderr, 
            "%s: in CheckAuthorizedHost(): can't open '%s': error %d: %s\n", 
            program, path, errno, strerror(errno)
        );
        return -1;
    }
    
    while(!feof(file)) {
        fgets(buf, BUF_SIZE, file);
        
        line = strchr(buf, '#');       // Erasing comments
        if(line != NULL) {
            *line = 0;
        }
        
        chomp(buf);
        
        if(strlen(buf) == 0) {         // Skipping empty lines
            continue;
        }
        
        line = buf;
        host = strchr(buf, ':');
        *host = 0;
        host++;
        comments = strchr(host, ':');
        *comments = 0;
        comments++;
        
        /*
        line = strtok(buf,  ":");      // Getting the line
        host = strtok(NULL, ":");      // Getting the host
        comments = strtok(NULL, ":");  // Getting the comments
        */
        if(match(host, hostname)) {
            if(only_new) {
                if (line[0] == 'N') {
                    printf("\033c\a"
                       " .----------------------------------------------------------. \n"
                       " | Sorry,  but this host  is not  authorized to  create new |.\n"
                       " | accounts on this server. Here is the reason:             ||\n"
                       " | %56s ||\n"
                       " `__________________________________________________________'|\n"
                       "   ----------------------------------------------------------'\n", 
                       comments
                    );
                    fclose(file);
                    exit(0);
                }
                
            } else {
                if(line[0] == 'X') {                    
                    printf("\033c\a"
                           " .----------------------------------------------------------. \n"
                           " | Sorry,  but this host  is not  authorized to  connect to |.\n"
                           " | this server. Here is the reason:                         ||\n"
                           " | %56s ||\n"
                           " `__________________________________________________________'|\n"
                           "   ----------------------------------------------------------'\n", 
                           comments
                    );
                    fclose(file);
                    exit(0);
                }
            }
        }
    }
    
    fclose(file);
    return(true);
}


/* ***************************************************************** *
 * Language()                                                        *
 *                                                                   *
 * ***************************************************************** */
char * Language(int language) {
    switch(language) {
        case L_fr: return "French";    break;
        case L_uk: return "English";   break;
        case L_sp: return "Spanish";   break;
        case L_it: return "Italian";   break;
        case L_no: return "Norvegian"; break;
        default:   return "";          break;
    }
}


/* ***************************************************************** *
 * SetTerminalMode()                                                 *
 * ---------------                                                   *
 * description                                                       *
 *   This function changes the terminal mode to the given mode.      *
 *   The only recognized modes are:                                  *
 *                                                                   *
 *     TERM_MODE_RAW                                                 *
 *         The terminal is in raw mode; input can be read character  *
 *         by character but none is echoed. It is up to the read     *
 *         function to echo the read characters, handle the          *
 *         backspace, etc.                                           *
 *                                                                   *
 *     TERM_MODE_SANE                                                *
 *         Restore the canonical terminal mode: input is read by     *
 *         line, characters are echoed, etc.                         *
 *                                                                   *
 * ***************************************************************** */
void SetTerminalMode(int mode) {
    
    tcgetattr(STDIN_FILENO, &Term);
    
    if(mode == TERM_MODE_RAW) {
        oldCtrl = Term.c_lflag;
        oldVMin = Term.c_cc[VMIN];
        Term.c_lflag &= !(ECHO|ICANON);
        Term.c_cc[VMIN] = 1;
        tcsetattr(STDIN_FILENO, TCSANOW, &Term);
    }
    
    if(mode == TERM_MODE_SANE) {
        Term.c_lflag = oldCtrl;
        Term.c_cc[VMIN] = oldVMin;
        tcsetattr(STDIN_FILENO, TCSANOW, &Term);
    }
}


/* ***************************************************************** *
 * Login::Login()                                                    *
 * ------------                                                      *
 * description                                                       *
 *   This is the Login object constructor.                           *
 *                                                                   *
 * ***************************************************************** */
Login::Login(void) {
    // Mandatory properties.
    login    = new char[LG_NOM_USER + 1];
    usernum  = 0;
    passwd   = new char[MAX_LEN + 1];
    email    = new char[MAX_LEN + 1];
    language = L_uk;
    key      = GetKey();
    
    // Optional properties.
    name     = new char[MAX_LEN + 1];
}


/* ***************************************************************** *
 * Login::~Login()                                                   *
 * -------------                                                     *
 * description                                                       *
 *   This is the Login object destructor.                            *
 *                                                                   *
 * ***************************************************************** */
Login::~Login(void) {
    // Mandatory properties.
    delete [] login;
    delete [] passwd;
    delete [] email;
    
    // Optional properties.
    delete [] name;
}


/* ***************************************************************** *
 * Login::Logon()                                                    *
 * ------------                                                      *
 * description                                                       *
 *   This is the login menu.                                         *
 *   The return code indicate what to do:                            *
 *     0 => the login name is not valid                              *
 *     1 => the login name is valid                                  *
 *     2 => the user must be created                                 *
 *                                                                   *
 * ***************************************************************** */
int Login::Logon(void) {
    String  user;
    FILE *  file = NULL;
    char *  path = NULL;
    char    essay[MAX_LEN+1];   // buffer for the password
    int     rc = 0;             // return code
    
    printf("Login: ");
    GetName(login, LG_NOM_USER);
    
    // Login: Off
    if(strcmp(login, "Off") == 0) {
        printf("Have a nice day.\n");
        exit(0);
    }
    
    // Login: New
    if(strcmp(login, "New") == 0) {
        return 2;
    }
    
    // Login: About
    if(strcmp(login, "About") == 0) {
        About();
        return 0;
    }
    
    // Login: Help
    if(strcmp(login, "Help") == 0) {
        Help();
        return 0;
    }
    
    // Login: Version
    if(strcmp(login, "Version") == 0) {
        printf("%s version %s\n", program, version);
        return 0;
    }
    
    // Valid name; testing if the user exists
    user = String(login);
    path = G.GetUserFile(user, User_Passwd_file);
    file = fopen(path, "r");
    
    if(file != NULL) {
        // The user exists. 
        // Now the user must enter his/her password.
        
        fgets(passwd, MAX_LEN, file);
        chomp(passwd);
        
        strcpy(essay, GetPassword("Password: "));
        
        if(strcmp(passwd, essay) == 0) {
            // The password is correct.
            rc = 1;
            
        } else {
            // The password is wrong.
            printf("Wrong password. Try again.\n");
            rc = 0;
        }
        
    } else {
        printf("This user name doesn't exist.\n");
        rc = 0;
    }
    
    return rc;
}


/* ***************************************************************** *
 * Login::Help()                                                     *
 * -----------                                                       *
 * description                                                       *
 *   This function prints a help screen.                             *
 *                                                                   *
 * ***************************************************************** */
void Login::Help(void) {
    printf("\033c\n"
           " .----------------------------------------------------------. \n"
           " |                      GNU's Pipo-BBS                      |.\n"
           " |                                                          ||\n"
           " | Your are currently  connected with  Atlantis BBS,  a BBS ||\n"
           " | running  GNU's  Pipo-BBS.  You can type  'About'  at the ||\n"
           " | login prompt to obtain information about this software.  ||\n"
           " |                                                          ||\n"
           " | Enter your login name  and your password to connect.  If ||\n"
           " | don't have any account,  type 'New'  and then follow the ||\n"
           " | creation processus. If you don't remember your password, ||\n"
           " | you can use the Guest account (login: Guest, password:   ||\n"
           " | guest)  which will  allow you  to connect  and Yell  the ||\n"
           " | administrators in order to explain them your problem.    ||\n"
           " |                                                          ||\n"
           " | If you can't log on  even when you have  correctly typed ||\n"
           " | your login name and password, you can mail to one of the ||\n"
           " | administrators below.                                    ||\n"
           " |                                                          ||\n"
           " |   vdb@pipo.com                                           ||\n"
           " |   saper@esil.univ-mrs.fr                   The Atlantis  ||\n"
           " |   nort@pipo.com                              BBS Team    ||\n"
           " `__________________________________________________________'|\n"
           "   ----------------------------------------------------------'\n"
           "\n"
    );
}


/* ***************************************************************** *
 * Login::About()                                                    *
 * ------------                                                      *
 * description                                                       *
 *   This function prints an about screen.                           *
 *                                                                   *
 * ***************************************************************** */
void Login::About(void) {
    printf("\033c\n"
           " .----------------------------------------------------------. \n"
           " |                      GNU's Pipo-BBS                      |.\n"
           " |                                                          ||\n"
           " | This program is free software;  you can  redistribute it ||\n"
           " | and/or  modify it  under the terms  of  the  GNU General ||\n"
           " | Public  License  as  published   by  the  Free  Software ||\n"
           " | Foundation;  either version 2,  or (at your option)  any ||\n"
           " | later version.                                           ||\n"
           " | This program is distributed  in the hope that it will be ||\n"
           " | useful,  but  WITHOUT  ANY  WARRANTY;  without  even the ||\n"
           " | implied warranty  of  MERCHANTABILITY  or  FITNESS FOR A ||\n"
           " | PARTICULAR PURPOSE.  See the  GNU General Public License ||\n"
           " | for more details.                                        ||\n"
           " |                                                          ||\n"
           " | You can obtain  this program  from the  Pipo-BBS Project ||\n"
           " | ftp site <ftp://bbs.resus.org/pub>                       ||\n"
           " | You can obtain more information about  Atlantis BBS  and ||\n"
           " | GNU's Pipo-BBS at <http://bbs.resus.org/>                ||\n"
           " |                                                          ||\n"
           " | Copyright (C) 1997-1998 Free Software Foundation, Inc.   ||\n"
           " |                                                          ||\n"
           " |                                   The Atlantis BBS Team  ||\n"
           " `__________________________________________________________'|\n"
           "   ----------------------------------------------------------'\n"
           "\n"
    );
}


/* ***************************************************************** *
 * Login::Launch()                                                   *
 * -------------                                                     *
 * description                                                       *
 *   This function launches the client.                              *
 *                                                                   *
 * ***************************************************************** */
void Login::Launch(void) {
    char *  client_path;
    
    printf("\033c\n");
    client_path = G.GetClientApp();
    execlp(client_path, G.GetClientApp(), login, hostname, NULL);
    
    // If we are here, that means the exec() call failed. 
    fprintf(stderr, 
        "%s: execlp() failed: error %d: %s\n", 
        program, errno, strerror(errno)
    );
}


/* ***************************************************************** *
 * Login::NewUser()                                                  *
 * --------------                                                    *
 * description                                                       *
 *   This functions allows a user to create a new user account.      *
 *                                                                   *
 * ***************************************************************** */
void Login::NewUser(void) {
    
    FILE *  file = fopen(G.GetBinHome(Bbs_NoNew_file), "r");
    int     needed = 1;
    
    if(file != NULL) {
        printf("\033c\n"
               " .----------------------------------------------------------. \n"
               " | We are sorry, but the creation of new users is currently |.\n"
               " | desactivated. This is most probably because we are doing ||\n"
               " | some administration task.                                ||\n"
               " |                                                          ||\n"
               " | Please try to connect again in few minutes.              ||\n"
               " `__________________________________________________________'|\n"
               "   ----------------------------------------------------------'\n"
        );
        fclose(file);
        sleep(SLEEP_TIME);
        return;
    }
    
    CheckAuthorizedHost(true);
    printf("\033c\n"
           " .----------------------------------------------------------. \n"
           " | You are creating a new user account.                     |.\n"
           " |                                                          ||\n"
           " | We will lead you  during this process  and prompt you to ||\n"
           " | tell us some  informations we need  in order to create a ||\n"
           " | new user.  Note that  once  you have  created  your user ||\n"
           " | account,  you will be able to hide  all of your personal ||\n"
           " | informations.                                            ||\n"
           " |                                                          ||\n"
           " |                                   The Atlantis BBS Team  ||\n"
           " `__________________________________________________________'|\n"
           "   ----------------------------------------------------------'\n"
           "                  <Press Enter to continue>                   \n"
    );

    getc(stdin);
    printf("\033c\n");
    
    while(needed) {
        
        try {
            Nickname();
            Passwd();
            Email();
            ChooseLang();
            RealName();
            ConfirmInfos();
            WriteData();
            Launch();
        }
        
        catch(UserCancelled Err) {
            // The user cancelled, therefore we 
            // cycle the process. 
            continue;
        }
        
        catch(CreationError Err) {
            // There was an error. 
            continue;
        }
    }
}


/* ***************************************************************** *
 * Login::Clean()                                                    *
 * ------------                                                      *
 * description                                                       *
 *   This functions removes the user's files.                        *
 *                                                                   *
 * ***************************************************************** */
void Login::Clean(void) {
    String  user(login);
    
    remove( G.GetUserFile(user, User_Infos_file ) );
    remove( G.GetUserFile(user, User_Passwd_file) );
}


/* ***************************************************************** *
 * Login::Nickname()                                                 *
 * ---------------                                                   *
 * description                                                       *
 *   This is the nickname setting menu.                              *
 *                                                                   *
 * note                                                              *
 *   All the proper names must obey to the following scheme:         *
 *    - only alphabetic and space characters can be used             *
 *    - every single character following a space must be upper case  *
 *   Therefore this is done at the user input in order to keep the   *
 *   whole thing simple in appearance.                               *
 *                                                                   *
 * ***************************************************************** */
void Login::Nickname(void) {
    String  user;
    FILE *  file = NULL;
    char *  path = NULL;
    int     needed = 1;
    
    printf("\n"
           " .----------------------------------------------------------. \n"
           " | You must choose a nickname.                              |.\n"
           " |                                                          ||\n"
           " | This nickname will be the name  everyone on the bbs will ||\n"
           " | use to speak with you.  Note that for technical reasons, ||\n"
           " | only alphabetics and space characters are allowed in the ||\n"
           " | nickname  (i.e. the letters  'a' to 'z'  and  'A' to 'Z' ||\n"
           " | plus the space). Type 'Off' to abort.                    ||\n"
           " `__________________________________________________________'|\n"
           "   ----------------------------------------------------------'\n"
    );
    
    while(needed) {
        needed = 0;
        printf("Please enter a user name: ");
        GetName(login, LG_NOM_USER);
        
        // Off
        if(strcmp(login, "Off") == 0) {
            printf("Creation process cancelled.\nHave a nice day.\n");
            exit(0);
        }
        
        // Testing if the user exists.
        user = String(login);
        path = G.GetUserFile(user, User_Passwd_file);
        file = fopen(path, "r");
        
        if(file != NULL) {
            fclose(file);
            needed = 1;
            printf(" .----------------------------------------------------------. \n"
                   " | Sorry, but this nickname is already used.                |.\n"
                   " | Please choose another nickname and try again.            ||\n"
                   " `__________________________________________________________'|\n"
                   "   ----------------------------------------------------------'\n"
            );
            continue;
        }
        
        // Testing if the name is allowed.
        if(Taboo(login)) {
            needed = 1;
            printf(" .----------------------------------------------------------. \n"
                   " | Sorry, but this nickname is not allowed.                 |.\n"
                   " | Please choose another nickname and try again.            ||\n"
                   " `__________________________________________________________'|\n"
                   "   ----------------------------------------------------------'\n"
            );
            continue;
        }
    }
    
    printf("\n");
}


/* ***************************************************************** *
 * Login::Passwd()                                                   *
 * -------------                                                     *
 * description                                                       *
 *   This is the password setting menu.                              *
 *                                                                   *
 * ***************************************************************** */
void Login::Passwd(void) {
    char    essay_1[MAX_LEN + 1] = "1";
    char    essay_2[MAX_LEN + 1] = "2";
    int     needed = 1;
    
    printf("\n"
           " .----------------------------------------------------------. \n"
           " | Now you must choose a password.                          |.\n"
           " |                                                          ||\n"
           " | This password is confidential,  therefore  you shouldn't ||\n"
           " | give it to third persons.  We recommand you  to choose a ||\n"
           " | password  that will  be easy  for you  to remember,  but ||\n"
           " | which shouldn't figure in a dictionary;  this can be for ||\n"
           " | example the first letters of the words of a sentence, or ||\n"
           " | a word without vowels. Type ctrl-C to cancel.            ||\n"
           " `__________________________________________________________'|\n"
           "   ----------------------------------------------------------'\n"
    );
    
    // We loop the password double entry until the two tries match.
    
    while(needed) {
        
        strcpy(essay_1, GetPassword("Please enter a password: "));
        
        if(strlen(essay_1) < PASSWORD_MIN_LENGTH) {
            printf(" .----------------------------------------------------------. \n"
                   " | This password is too short.                              |.\n"
                   " | For security reasons, your password must be at least %d   ||\n"
                   " | characters long.                                         ||\n"
                   " `__________________________________________________________'|\n"
                   "   ----------------------------------------------------------'\n", 
                   PASSWORD_MIN_LENGTH
            );
            continue;
        }
        
        strcpy(essay_2, GetPassword("Please re-enter your password: "));
        
        if(strcmp(essay_1, essay_2) != 0) {
            needed = 1;
            printf(" .----------------------------------------------------------. \n"
                   " | The two entries  are not identical;  you probably made a |.\n"
                   " | mistake. Please try again.                               ||\n"
                   " `__________________________________________________________'|\n"
                   "   ----------------------------------------------------------'\n"
            );
        } else {
            needed = 0;
        }
    }
    
    // We save the password. 
    strcpy(passwd, essay_1);
    
    printf("\n");
}


/* ***************************************************************** *
 * Login::Email()                                                    *
 * ------------                                                      *
 * description                                                       *
 *   This is the email address setting menu.                         *
 *                                                                   *
 * ***************************************************************** */
void Login::Email(void) {
    int     needed = 1;
    //int     len = 0;
    //int     i = 0;
    
    printf("\n"
           " .----------------------------------------------------------. \n"
           " | You must give us your email adress.                      |.\n"
           " |                                                          ||\n"
           " | Please note  that this is  mandatory  for administration ||\n"
           " | purpose,  and that you can hide  this information  from  ||\n"
           " | other users if you want.                                 ||\n"
           " `__________________________________________________________'|\n"
           "   ----------------------------------------------------------'\n"
    );
    
    while(needed) {
        needed = 0;
        printf("Please enter your email address: ");
        GetString(email, MAX_LEN);
        chomp(email);
        
        if(CheckEmailAddress(email) == 0) {
            needed = 1;
            printf(" .----------------------------------------------------------. \n"
                   " | Sorry, but this email address doesn't seem valid.        |.\n"
                   " | Please check and try again.                              ||\n"
                   " `__________________________________________________________'|\n"
                   "   ----------------------------------------------------------'\n"
            );
            continue;
        }
    }
    
    printf("\n");
}


/* ***************************************************************** *
 * Login::ChooseLang()                                               *
 * -----------------                                                 *
 * description                                                       *
 *   This is the language option menu.                               *
 *                                                                   *
 * ***************************************************************** */
void Login::ChooseLang(void) {
    int     needed = 1;
    char    cmd = 0;
    
    printf("\n"
           " .----------------------------------------------------------. \n"
           " | This bbs have a multilanguage system.                    |.\n"
           " | Please choose your language and press Enter.             ||\n"
           " `__________________________________________________________'|\n"
           "   ----------------------------------------------------------'\n"
    );
    
    while(needed) {
        printf("     <1>(%c)French  <2>(%c)English <3>(%c)Spanish \n"
               "     <4>(%c)Italian <5>(%c)Norvegian              \n"
               "Your choice: ", 
               star(language == L_fr), star(language == L_uk), 
               star(language == L_sp), star(language == L_it), 
               star(language == L_no)
        );
        
        cmd = getc(stdin);
        printf("\n");
        
        switch(cmd) {
            case '1':  language = L_fr; break;
            case '2':  language = L_uk; break;
            case '3':  language = L_sp; break;
            case '4':  language = L_it; break;
            case '5':  language = L_no; break;
            case '\n': needed = 0;      break;
        }
        
    }
    
    printf("\n");
}


/* ***************************************************************** *
 * Login::RealName()                                                 *
 * ---------------                                                   *
 * description                                                       *
 *   This menu is for setting the real user name.                    *
 *                                                                   *
 * ***************************************************************** */
void Login::RealName(void) {
    char    name1[MAX_LEN+1];
    char    name2[MAX_LEN+1];
    int     needed = 1;
    
    printf("\n"
           " .----------------------------------------------------------. \n"
           " | Now you can enter your real name.                        |.\n"
           " |                                                          ||\n"
           " | Please note that this is not mandatory,  and that you'll ||\n"
           " | have the possibility to hide this information.           ||\n"
           " `__________________________________________________________'|\n"
           "   ----------------------------------------------------------'\n"
    );
    
    needed = 1;
    while(needed) {
        printf("First Name(s): ");
        GetString(name1, MAX_LEN);
        chomp(name1);
        if(strlen(name1) < FIRST_NAME_MIN_LEN) {
            printf(" .----------------------------------------------------------. \n"
                   " | Sorry, but this entry doesn't seem valid.                |.\n"
                   " | Please try again.                                        ||\n"
                   " `__________________________________________________________'|\n"
                   "   ----------------------------------------------------------'\n"
            );
            continue;
        }
        needed = 0;
    }
    
    needed = 1;
    while(needed) {
        printf("Last Name: ");
        GetString(name2, MAX_LEN);
        chomp(name2);
        if(strlen(name2) < FIRST_NAME_MIN_LEN) {
            printf(" .----------------------------------------------------------. \n"
                   " | Sorry, but this entry doesn't seem valid.                |.\n"
                   " | Please try again.                                        ||\n"
                   " `__________________________________________________________'|\n"
                   "   ----------------------------------------------------------'\n"
            );
            continue;
        }
        needed = 0;
    }
    
    strcpy(name, name1);
    strcat(name, " ");
    strcat(name, name2);
    
    printf("\n");
}


/* ***************************************************************** *
 * Login::ConfirmInfos()                                             *
 * ------------------                                                *
 * description                                                       *
 *   This function show a summary of the informations s/he submitted *
 *   and asks him/her it every thing is correct.                     *
 *                                                                   *
 * ***************************************************************** */
void Login::ConfirmInfos(void) {
    printf("\n"
           " .----------------------------------------------------------. \n"
           " | Here is a summary of the informations you give us.       |.\n"
           " |                                                          ||\n"
           " | If there is any mistake in one of your informations,     ||\n"
           " | answer \"No\" to the question and we will try again.       ||\n"
           " `__________________________________________________________'|\n"
           "   ----------------------------------------------------------'\n"
    );
    
    printf("\n"
           "    Nickname:   %s\n"
           "    Email:      %s\n"
           "    Language:   %s\n"
           "    Real name:  %s\n",
           login, 
           email, 
           Language(language), 
           name
    );
    
    printf("\nAre these informations correct so far ? (Yes/No) ");
    
    while(1) {
        switch(getc(stdin)) {
            case 'y':
            case 'Y':
                puts("Yes");
                return;
            
            case 'n':
            case 'N':
                puts("No");
                throw UserCancelled();
                return;
            
            default: 
                continue;
        }
    }
}


/* ***************************************************************** *
 * Login::WriteData()                                                *
 * ----------------                                                  *
 * description                                                       *
 *   This function writes the user data to the disk.                 *
 *                                                                   *
 * ***************************************************************** */
int Login::WriteData(void) {
    String  user(login);
    FILE *  file = NULL;
    char *  path = NULL;
    char    dir[512];
    
    // Creating the directories structure --
    
    strcpy(dir, G.GetUsrHome(""));
    strcat(dir, "/");
    strncat(dir, login, 2);
    strcat(dir, "/");
    
    if(mkdir(dir, 0700) && errno != EEXIST) {
        fprintf(stderr, 
            "%s: in WriteData(): can't create directory '%s': error %d: %s\n", 
            program, dir, errno, strerror(errno)
        );
        
        throw CreationError();
    }
    
    strcat(dir, login);
    
    if(mkdir(dir, 0700) && errno != EEXIST) {
        fprintf(stderr, 
            "%s: in WriteData(): can't create directory '%s': error %d: %s\n", 
            program, dir, errno, strerror(errno)
        );
        
        throw CreationError();
    }
    
    // Writing general informations --
    
    usernum = UserNumber();
    
    if(usernum < 0) {
        throw CreationError();
    }
    
    path = G.GetUserFile(user, User_Infos_file);
    
    file = fopen(path, "w");
    
    if(file == NULL) {
        fprintf(stderr, 
            "%s: in WriteData(): can't open '%s': error %d: %s\n", 
            program, path, errno, strerror(errno)
        );
        return 0;
    }
    
    fprintf(file, "num   = %d\n",  usernum  );
    fprintf(file, "nom   = %s\n",  name     );
    fprintf(file, "mail  = %s\n",  email    );
    fprintf(file, "mere  = %u\n",  language );
    fprintf(file, "droit = 1\n"             );
    fprintf(file, "first = %ld\n", GetTime());
    //fprintf(file, "cle   = %d\n",  key      );
    fclose(file);
    
    // Writing user password --
    
    path = G.GetUserFile(user, User_Passwd_file);
    
    file = fopen(path, "w");
    
    if(file == NULL) {
        fprintf(stderr, 
            "%s: in WriteData(): can't open '%s': error %d: %s\n", 
            program, path, errno, strerror(errno)
        );
        
        throw CreationError();
    }
    
    fprintf(file, "%s\n", passwd);
    fclose(file);
    
    printf("\n"
           " .----------------------------------------------------------. \n"
           " | Your user has been successfully created.                 |.\n"
           " |                                                          ||\n"
           " | The Atlantis BBS Team hope you'll enjoy being on board.  ||\n"
           " | Remember that if you need help,  you can read the online ||\n"
           " | help files with the key <h>, and ask a question with the ||\n"
           " | key <Q>.                                                 ||\n"
           " `__________________________________________________________'|\n"
           "   ----------------------------------------------------------'\n"
           "                  <Press Enter to continue>                   \n"
    );
    
    getc(stdin);
    
    return 1;
}


/* ***************************************************************** *
 * Login::UserNumber()                                               *
 * -----------------                                                 *
 * description                                                       *
 *   This function returns a new user number and updates the users   *
 *   list. Therefore this function can only be called once a valid   *
 *   login is stored in the login[] string.                          *
 *                                                                   *
 * ***************************************************************** */
int Login::UserNumber(void) {
    FILE *  file = NULL;
    char *  path = NULL;
    char    buf[BUF_SIZE+1];
    int     num  = 0;
    
    // Reading the new user number from the users list --
    
    path = G.GetEtcHome(Users_List_file);
    
    file = fopen(path, "r");
    
    if(file == NULL) {
        fprintf(stderr, 
            "%s: in UserNumber(): can't open '%s': error %d: %s\n", 
            program, path, errno, strerror(errno)
        );
        return -1;
    }
    
    while(!feof(file)) {
        fgets(buf, sizeof(buf) - 1, file);
        num++;
    }
    
    fclose(file);
    num--;
    
    // Updating the users list --
    
    file = fopen(path, "a");
    
    if(file == NULL) {
        fprintf(stderr, 
            "%s: in UserNumber(): can't open '%s': error %d: %s\n", 
            program, path, errno, strerror(errno)
        );
        return -1;
    }
    
    fprintf(file, "%s\n", login);
    
    fclose(file);
    
    return num;
}


/* ***************************************************************** *
 * Login::GetName()                                                  *
 * --------------                                                    *
 * description                                                       *
 *   This function stores in str[] the user input, formatted to be   *
 *   a valid name (len is the maximum length).                       *
 *                                                                   *
 * explanation                                                       *
 *   A valid names must follow these rules:                          *
 *    - only alphabetic and space characters can be used             *
 *    - every single character following a space must be upper case  *
 *   Therefore this is done at the user input in order to keep the   *
 *   whole thing simple in appearance.                               *
 *                                                                   *
 * ***************************************************************** */
int Login::GetName(char * str, int len) {
    char    c = 0;
    int     i = 0;
    int     must_be_upper = 1;
    int     space_allowed = 0;
    
    
    while(i <= len) {
        c = getc(stdin);
        
        // Context tests
        if(i == 0 || (i > 0 && str[i-1] == SPC)) {
            must_be_upper = 1;
            space_allowed = 0;
        }
        
        if(i > 0 && isalpha(str[i-1])) {
            must_be_upper = 0;
            space_allowed = 1;
        }
        
        // Backspace
        if(c == BS || c == DEL) {
            if(i > 0) {
                str[i] = 0;
                i--;
                str[i] = 0;
                putc(BS,  stdout);
                putc(SPC, stdout);
                putc(BS,  stdout);
            }
            continue;
        }
        
        // Enter
        if(c == CR || c == LF) {
            // Avoiding leading spaces
            if((space_allowed == 0) && (i>0)) {
                str[i-1] = 0;
            } else {
                str[i] = 0;
            }
            putc(CR, stdout);
            putc(LF, stdout);
            break;
        }
        
        // Cancel (ctrl-c or ctrl-d)
        if(c == ETX || c == EOT) {
            throw UserCancelled();
        }
        
        // End of buffer
        if(i == len) {
            putc('\a', stdout);
            continue;  // -> looping
        }
        
        // Space
        if(c == SPC) {
            if(space_allowed) {
                str[i] = SPC;
                i++;
                putc(SPC, stdout);
                must_be_upper = 1;
                space_allowed = 0;
                continue;
            }
            
            tcflush(0, TCIFLUSH);
            continue;
        }
        
        // Non-alpabetic character
        if(!isalpha(c) && c != SPC) {
            putc('\a', stdout);
            tcflush(0, TCIFLUSH);
            continue;
        }
        
        // Alphabetic character
        if(islower(c) && must_be_upper) {
            str[i] = toupper(c);
            
        } else {
            str[i] = c;
        }
        
        putc(str[i], stdout); 
        must_be_upper = 0;
        space_allowed = 1;
        i++;
    }
    
    str[i] = 0;
    
    return i;
}


/* ***************************************************************** *
 * Login::GetPassword()                                              *
 * ------------------                                                *
 * description                                                       *
 *   This function prompts for and returns a password.               *
 *                                                                   *
 * explanation                                                       *
 *   Each typed character is echoed by a star '*'. This is done      *
 *   using the terminal in TERM_MODE_RAW mode.                       *
 *                                                                   *
 * note                                                              *
 *   This function has exactly the same prototype as the Unix        *
 *   getpass() function, and quite the same behaviour, except in     *
 *   the fact that no flush is done. This detail is important        *
 *   because users with laggy connection may experiment trouble when *
 *   typing their password. Also, echoing a dummy character rather   *
 *   than nothing is easier to use for novice users (and BBS users   *
 *   are generally not very familiar with computers).                *
 *                                                                   *
 * ***************************************************************** */
char * Login::GetPassword(char * prompt) {
    char *  pass = new char [BUF_SIZE + 1];
    char    c = 0;
    int     i = 0;
    
    printf(prompt);
    
    while(i <= BUF_SIZE) {
        c = getc(stdin);
        
        // Backspace
        if(c == BS || c == DEL) {
            if(i > 0) {
                pass[i] = 0;
                i--;
                pass[i] = 0;
                putc(BS,  stdout);
                putc(SPC, stdout);
                putc(BS,  stdout);
            }
            continue;
        }
        
        // Enter
        if(c == CR || c == LF) {
            pass[i] = 0;
            putc(CR, stdout);
            putc(LF, stdout);
            break;
        }
        
        // Cancel (ctrl-c or ctrl-d)
        if(c == ETX || c == EOT) {
            throw UserCancelled();
        }
        
        // End of buffer
        if(i == BUF_SIZE) {
            putc('\a', stdout);
            continue;  // -> looping
        }
        
        pass[i] = c;
        putc(PASSWORD_CHAR, stdout);
        i++;
    }
    
    pass[i] = 0;
    
    return pass;
}


/* ***************************************************************** *
 * Login::GetString()                                                *
 * ----------------                                                  *
 * description                                                       *
 *   This function stores in str[] the user input.                   *
 *                                                                   *
 * explanation                                                       *
 *   This function is quite similar to fgets(), excepts that it      *
 *   correctly treats both delete and backspace characters.          *
 *                                                                   *
 * ***************************************************************** */
int Login::GetString(char * str, int len) {
    char    c = 0;
    int     i = 0;
    
    
    while(i <= len) {
        c = getc(stdin);
        
        // Backspace
        if(c == BS || c == DEL) {
            if(i > 0) {
                str[i] = 0;
                i--;
                str[i] = 0;
                putc(BS,  stdout);
                putc(SPC, stdout);
                putc(BS,  stdout);
            }
            continue;
        }
        
        // Enter
        if(c == CR || c == LF) {
            str[i] = 0;
            putc(CR, stdout);
            putc(LF, stdout);
            break;
        }
        
        // Cancel (ctrl-c or ctrl-d)
        if(c == ETX || c == EOT) {
            throw UserCancelled();
        }
        
        // End of buffer
        if(i == len) {
            putc('\a', stdout);
            continue;  // -> looping
        }
        
        str[i] = c;
        putc(str[i], stdout);
        i++;
    }
    
    str[i] = 0;
    
    return i;
}


/* ***************************************************************** *
 * Login::GetTime()                                                  *
 * --------------                                                    *
 * description                                                       *
 *   This function returns the current time.                         *
 *                                                                   *
 * ***************************************************************** */
time_t Login::GetTime(void) {
    return time(NULL);
}


/* ***************************************************************** *
 * Login::GetKey()                                                   *
 * -------------                                                     *
 * description                                                       *
 *   This function returns a new key.                                *
 *                                                                   *
 * ***************************************************************** */
int Login::GetKey(void) {
    int     key = 0;
    
    srand(time(0));
    key = rand() % 10000;
    
    return key;
}


/* ***************************************************************** *
 * Login::GetHostname()                                              *
 * ------------------                                                *
 * description                                                       *
 *   This function returns the hostname of the remote machine.       *
 *                                                                   *
 * explanation                                                       *
 *   To find the 
 *                                                                   *
 * ***************************************************************** */
char * Login::GetHostname(void) {
    /*
    FILE *  file = NULL;
    char    hostname[MAX_LEN+1];
    
    file = fopen(UTMP_FILE, "r");
    
    if(file == NULL) {
        fprintf(stderr, 
            "%s: in GetHostname(): can't open '%s': error %d: %s\n", 
            program, path, errno, strerror(errno)
        );
        return NULL;
    }
    
    
    */
    return NULL;
}


/* ***************************************************************** *
 * Login::Taboo()                                                    *
 * ------------                                                      *
 * description                                                       *
 *   This function returns 1 if the given name is a taboo word,      *
 *   zero otherwise.                                                 *
 *                                                                   *
 * ***************************************************************** */
int Login::Taboo(char * name) {
    FILE *  file = NULL;
    char *  path = NULL;
    char *  low_name = NULL;
    char    word[MAX_LEN+1];
    int     is_taboo = 0;
    
    
    low_name = new char[strlen(name) + 1];
    strcpy(low_name, name);
    lower_case(low_name);
    
    path = G.GetEtcHome(Taboo_file);
    
    file = fopen(path, "r");
    
    if(file == NULL) {
        fprintf(stderr, 
            "%s: in Taboo(): can't open '%s': error %d: %s\n", 
            program, path, errno, strerror(errno)
        );
        return 0;
    }
    
    while(!feof(file)) {
        fscanf(file, "%s", word);
        
        if(word[0] == '@' || word[0] == '.') {
            is_taboo = 0;
            break;
        }
        
        lower_case(word);
        
//     printf("comparing '%s' and '%s'\n", low_name, word);
        if(strstr(word, low_name) != NULL) {
            is_taboo = 1;
//     puts("  => match");
            break;
        }
    }
    
    fclose(file);
    delete [] low_name;
    
    return is_taboo;
}


/* ***************************************************************** *
 * Login::CheckEmailAddress()                                        *
 * ------------------------                                          *
 * description                                                       *
 *   This function checks the validity of the given email address,   *
 *   and returns 1 if is seems valid, zero otherwise.                *
 *                                                                   *
 * explanation                                                       *
 *   Here is what points are checked:                                *
 *    - is there an arobas ? a point ?                               *
 *    - is the email long enough ?                                   *
 *    - are the two parts (part1@part2) long enough ?                *
 *                                                                   *
 * ***************************************************************** */
int Login::CheckEmailAddress(char * email) {
    char *  s = NULL;
    int     i = 0;
    int     len = strlen(email);
    int     Llen = 0;
    int     Rlen = 0;
    int     at_sign_here = 0;
    int     point_here = 0;
    
    if(len < EMAIL_MIN_LEN) {
        return 0;
    }
    
    for(i=0; i<len; i++) {
        if(email[i] == '@')  at_sign_here = 1;
        if(email[i] == '.')  point_here = 1;
        
        // Typos auto-correction
        if(email[i] == ';')  email[i] = '.';
        if(email[i] == '|')  email[i] = ' ';
        if(email[i] == '&')  email[i] = ' ';
        if(email[i] == '$')  email[i] = ' ';
        if(email[i] == '%')  email[i] = ' ';
        if(email[i] == '\\') email[i] = ' ';
    }
    
    if(at_sign_here) {
        s = strchr(email, '@');
        Rlen = strlen(s) - 1;
        Llen = len - Rlen;
        if(Llen < FIRST_NAME_MIN_LEN)  return 0;
        if(Rlen < EMAIL_MIN_LEN - FIRST_NAME_MIN_LEN)  return 0;
    }
    
    return (at_sign_here && point_here);
}


/* ***************************************************************** *
 * Login::SendEmail()                                                *
 * ----------------                                                  *
 * description                                                       *
 *   This function sends the given mail to the given email address.  *
 *                                                                   *
 * ***************************************************************** */
int Login::SendEmail(char * address, char * message) {
    Mail    mail;
    
    CheckEmailAddress(address);
    
    mail.Sender("bbs@bbs.resus.univ-mrs.fr");
    mail.Recipient(address);
    mail.Subject("");
    mail.Message(message);
    mail.Send();
    
    return 1;
}


/* ***************************************************************** *
 * Login::SendValidationMail()                                       *
 * -------------------------                                         *
 * description                                                       *
 *   This function send a mail to the user with his/her validation   *
 *   key.                                                            *
 *                                                                   *
 * ***************************************************************** */
int Login::SendValidationMail(void) {
    char    msg[500];
    
    sprintf(msg, 
        "\n  Hello %s, \n\n"
        "  You have just created  a new user account  on  Atlantis BBS,\n"
        "the first  BBS  running  Gnu's Pipo-BBS.  This mail  has been\n"
        "automatically sent to you  in order to give you  your personal\n"
        "validation key. This is required to validate you user account.\n"
        "Your validation key is:\n"
        "\n    %d\n\n"
        "You just have to enter this key when it is prompted to you.\n\n"
        "  If you want more information  about  Atlantis BBS  or  Gnu's\n"
        "Pipo-BBS, please take a look at the following url:\n"
        "    http://bbs.resus.org\n\n"
        "  We hope you will like using Atlantis. \n"
        "\n              The Atlantis BBS Team\n", 
        login, key
    );
    
    return SendEmail(email, msg);
}


/* ***************************************************************** *
 * Login::SendPasswordFailureMail()                                  *
 * ------------------------------                                    *
 * description                                                       *
 *   This function send a mail to the user with his/her password.    *
 *                                                                   *
 * note                                                              *
 *   The passwd field was filled with the real user password in the  *
 *   Logon() function.                                               *
 *                                                                   *
 * ***************************************************************** */
int Login::SendPasswordFailureMail(void) {
    char    msg[500];
    
    sprintf(msg, 
        "\n  Hello %s, \n\n"
        "  This is an automatic mail.\n"
        "  You (or maybe someone else) recently failed to log on and\n"
        "were not able to enter the real password. Therefore we sent\n"
        "you your password, in case you have forgotten it. \n"
        "  Your password is : %s\n"
        "  If you didn't failed yourself to log on, that probably \n"
        "means that someone tried to use your account. Please Yell the\n"
        "Administrators if you think so. "
        "\n              The Atlantis BBS Team\n", 
        login, passwd
    );
    
    return SendEmail(email, msg);
}


