/* ***************************************************************** *
 * Send Mail                                                         *
 * ================================================================= *
 * Creation date . . . . . . . . . . . . . . . . . . Sat 16 Jan 1999 *
 * Modification date . . . . . . . . . . . . . . . . Mon 18 Jan 1999 *
 * ================================================================= *
 * Copyright (C) 1997-1998 Free Software Foundation, Inc.            *
 *                                                                   *
 * This program  is  free software;  you can  redistribute it and/or *
 * modify it  under the terms  of the GNU  General Public License as *
 * published by  the Free Software Foundation;  either version 2, or *
 * (at your option) any later version.                               *
 *                                                                   *
 * This program  is distributed in the hope  that it will be useful, *
 * but WITHOUT  ANY WARRANTY;  without even  the implied warranty of *
 * MERCHANTABILITY  or  FITNESS  FOR A  PARTICULAR PURPOSE.  See the *
 * GNU General Public License for more details.                      *
 *                                                                   *
 * You should have received a copy of the GNU General Public License *
 * along  with  this  program;  if not,  write to  the Free Software *
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.         *
 *                                                                   *
 * ================================================================= *
 * Authors                                                           *
 *   Sebastien APERGHIS-TRAMONI <saper@esil.univ-mrs.fr>             *
 *                                                                   *
 * If you find any bug, or if you do any improvment, please send us  *
 * a mail (possibly with diff and/or correction).                    *
 *                                                                   *
 * ***************************************************************** */


/* ***************************************************************** *
 * standard includes                                                 *
 * ***************************************************************** */
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <errno.h>


/* ***************************************************************** *
 * network includes                                                  *
 * ***************************************************************** */
//#include <sys/types.h>
//#include <sys/socket.h>
//#include <netdb.h>


/* ***************************************************************** *
 * custom includes                                                   *
 * ***************************************************************** */
#include "mail.h"


/* ***************************************************************** *
 * defines                                                           *
 * ***************************************************************** */
#define BUF_SIZE    1024


/* ***************************************************************** *
 * global variables                                                  *
 * ***************************************************************** */
static char         program[]               = "Mail";
static char         version[]               = "1.0.0";
static char         crlf[]                  = "\015\12";



/* ***************************************************************** *
 * Mail::Mail()                                                      *
 * ----------                                                        *
 * description                                                       *
 *   This is the Mail constructor.                                   *
 *                                                                   *
 * ***************************************************************** */
Mail::Mail(char * sender, char * recipient, char * subject, char * message) {
    if(sender != NULL && strlen(sender) > 0) {
        _sender = new char[strlen(sender)];
        strcpy(_sender, sender);
    } else {
        _sender = NULL;
    }
    
    if(recipient != NULL && strlen(recipient) > 0) {
        _recipient = new char[strlen(recipient)];
        strcpy(_recipient, recipient);
    } else {
        _recipient = NULL;
    }
    
    if(subject != NULL && strlen(subject) > 0) {
        _subject = new char[strlen(subject)];
        strcpy(_subject, subject);
    } else {
        _subject = NULL;
    }
    
    if(message != NULL && strlen(message) > 0) {
        _message = new char[strlen(message)];
        strcpy(_message, message);
    } else {
        _message = NULL;
    }
}


/* ***************************************************************** *
 * Mail::~Mail()                                                     *
 * -----------                                                       *
 * description                                                       *
 *   This is the Mail destructor.                                    *
 *                                                                   *
 * ***************************************************************** */
Mail::~Mail(void) {
    if(_recipient != NULL)  delete [] _recipient;
    if(_subject   != NULL)  delete [] _subject;
    if(_message   != NULL)  delete [] _message;
}


/* ***************************************************************** *
 * Mail::Server()                                                    *
 * ------------                                                      *
 * description                                                       *
 *   This is the server field access method.                         *
 *   If the argument is NULL, the server field is constructed using  *
 *   the recipient field (we take the string following the '@', as   *
 *   it should be).                                                  *
 *                                                                   *
 * ***************************************************************** */
void Mail::Server(char * server) {
    if(server == NULL) {
        char * s = strchr(_recipient, '@');
        if(s == NULL) throw Mail_MalformedAddress();
        s++;
        _server = new char[strlen(s)];
        strcpy(_server, s);
        return;
    }
    
    if(server != NULL && strlen(server) > 0) {
        if(_server != NULL) {
            delete [] _server;
        }
        _server = new char[strlen(server)];
        strcpy(_server, server);
    }
}


/* ***************************************************************** *
 * Mail::Sender()                                                    *
 * ------------                                                      *
 * description                                                       *
 *   This is the sender field access method.                         *
 *                                                                   *
 * ***************************************************************** */
void Mail::Sender(char * sender) {
    if(sender != NULL && strlen(sender) > 0) {
        if(_sender != NULL) {
            delete [] _sender;
        }
        _sender = new char[strlen(sender)];
        strcpy(_sender, sender);
    }
}


/* ***************************************************************** *
 * Mail::Recipient()                                                 *
 * ---------------                                                   *
 * description                                                       *
 *   This is the recipient field access method.                      *
 *                                                                   *
 * ***************************************************************** */
void Mail::Recipient(char * recipient) {
    if(recipient != NULL && strlen(recipient) > 0) {
        if(_recipient != NULL) {
            delete [] _recipient;
        }
        _recipient = new char[strlen(recipient)];
        strcpy(_recipient, recipient);
    }
}


/* ***************************************************************** *
 * Mail::Subject()                                                   *
 * -------------                                                     *
 * description                                                       *
 *   This is the subject field access method.                        *
 *                                                                   *
 * ***************************************************************** */
void Mail::Subject(char * subject) {
    if(subject != NULL && strlen(subject) > 0) {
        if(_subject != NULL) {
            delete [] _subject;
        }
        _subject = new char[strlen(subject)];
        strcpy(_subject, subject);
    }
}


/* ***************************************************************** *
 * Mail::Message()                                                   *
 * -------------                                                     *
 * description                                                       *
 *   This is the message field access method.                        *
 *                                                                   *
 * ***************************************************************** */
void Mail::Message(char * message) {
    if(message != NULL && strlen(message) > 0) {
        if(_message != NULL) {
            delete [] _message;
        }
        _message = new char[strlen(message)];
        strcpy(_message, message);
    }
}


/* ***************************************************************** *
 * Mail::Connect()                                                   *
 * -------------                                                     *
 * description                                                       *
 *   This function is used by the Send() function to connect to the  *
 *   remote mail server.                                             *
 *                                                                   *
 * exceptions                                                        *
 *   Mail_BadPortNumber                                              *
 *       The given port is not a valid port.                         *
 *                                                                   *
 *   Mail_ServerUnknown                                              *
 *       The DNS request to get the server's IP failed.              *
 *                                                                   *
 *   Mail_ServerUnreachable                                          *
 *       The connection to the server failed.                        *
 *                                                                   *
 * ***************************************************************** */
int Mail::Connect(char * server, int port) {
/*    struct sockaddr_in  addr;
    struct hostent *    host;
    int                 sock;
    
    if(port <= 0) {
        throw Mail_BadPortNumber();
    }
    
    host = gethostbyname(server);
    
    if(host == NULL) {
        throw Mail_ServerUnknown();
    }
    
    memset((char *)&addr, 0, sizeof(addr));
    memmove((char *)&addr.sin_addr, host->h_addr, host->h_length);
    addr.sin_family = host->h_addrtype;
    addr.sin_port = htons(port);
    
    sock = socket(AF_INET, SOCK_STREAM, 0);
    if( connect(sock,(struct sockaddr *)&addr, sizeof(addr)) ) {
        throw Mail_ServerUnreachable();
    }
    
    return sock;
    */
    return 1;
}


/* ***************************************************************** *
 * Mail::Send()                                                      *
 * ----------                                                        *
 * description                                                       *
 *   This function sends the mail.                                   *
 *                                                                   *
 * exceptions                                                        *
 *   Mail_NoRecipient                                                *
 *       No Recipient was given.                                     *
 *                                                                   *
 *   Mail_NoMessage                                                  *
 *       No message was given.                                       *
 *                                                                   *
 * ***************************************************************** */
void Mail::Send(void) {
    time_t  now = time(0);
    char    mail[4096];
    int     sock = 1;
    
    if(_recipient == NULL || strlen(_recipient) <= 1)  throw Mail_NoRecipient();
    if(_message   == NULL || strlen(_message  ) <= 1)  throw Mail_NoMessage();
    
    try {
        Server(0);
  printf( 
    "  Server: %s\n  Sender: %s\n  Recipient: %s\n  Subject: %s\n  Message: \n---\n%s---\n\n", 
    _server, _sender, _recipient, _subject, _message
  );
        sock = Connect(_server, 25);
    }
    
    catch(void * err) {
        throw err;
    }
    
    sprintf(mail, "%s%s%s%s%s%s%s%s%s%s%s%s%s%s"
            "Date: ", ctime(&now), crlf, 
            "From: ", "bbs@bbs.resus.univ-mrs.fr", crlf, 
            "Subject: ", _subject, crlf, 
            "To: ", _recipient, crlf, 
            crlf, _message
    );
    
    send(sock, mail, strlen(mail)+1, 0);
    
    return;
}


