#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <termios.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/wait.h>
#include <sys/time.h>
#include <arpa/inet.h>

#include "Client.H"

#define SERV_PORT "2000"
#define SERV_ADDR "139.124.29.1"

struct maillon_message_field;

struct maillon_message_field
        {
	char *field_name;
	char *field_value;
	struct maillon_message_field *next;
        };

struct message
        {
	char *msgtype;
	char *msgrange;
	struct maillon_message_field *msgbody;
        };

void afficher_msgbody (struct maillon_message_field *mai)
        {
	if (mai == NULL)
	        return;
	if ((mai->field_name == NULL) || (mai->field_value == NULL))
                return;
	printf ("\t%s = \"%s\"\n", mai->field_name, mai->field_value);
	afficher_msgbody (mai->next);
	}

void afficher_message (struct message *msg)
        {
	if (msg == NULL)
	        return;
	if ((msg->msgtype == NULL) || (msg->msgrange == NULL))
                return;
	printf ("Analyse du message :\n%c (%c)\n", *(msg->msgtype), *(msg->msgrange));
	if (msg->msgbody == NULL)
	        return;
	printf ("\t{\n");
	afficher_msgbody (msg->msgbody);
	printf ("\t}\n\n");
	}

void destroy_msgtype (char **msgtype)
        {
	if (*msgtype != NULL)
	        {
	      	free (*msgtype);
		*msgtype = NULL;
		}
	}

void destroy_msgrange (char **msgrange)
        {
	if (*msgrange != NULL)
	        {
	      	free (*msgrange);
		*msgrange = NULL;
		}
	}

void destroy_string (char **maillon)
        {
	if (*maillon != NULL)
	        {
		free (*maillon);
		*maillon = NULL;
		}
	}

void destroy_maillon (struct maillon_message_field **maillon)
        {
	if (*maillon != NULL)
	        {
		destroy_string (&((*maillon)->field_name));
		destroy_string (&((*maillon)->field_value));
		free ((*maillon)->next);
		*maillon = NULL;
		}
	}

void destroy_msgbody (struct maillon_message_field **msgbody)
        {
	if (*msgbody != NULL)
	        {
		destroy_msgbody (&((*msgbody)->next));
		destroy_maillon (msgbody);
		*msgbody = NULL;
		}
	}
	
void destroy_message (struct message **message)
        {
	if (*message != NULL)
	        {
		destroy_msgtype (&((*message)->msgtype));
		destroy_msgrange (&((*message)->msgrange));
		destroy_msgbody (&((*message)->msgbody));
		free (*message);
		*message = NULL;
		}
	}

int next_token (char *buffer)
        {
	int i = 0;

	while ((buffer [i] == ' ') || (buffer [i] == '\n') || (buffer [i] == '\t'))
		i++;

	return i;
	}

int chars (char c)
        {
	if (((c >= 'a') && (c <= 'z')) || ((c >= 'A') && (c <= 'Z')))
	        return 1;
	return 0;
	}

void new_token (char *buffer, char *ptoken, int *token_length)
        {
	int i = 1;
	int j;
	int cont  = 1;
	int last_was_slash = 0;
	int k = 0;

	if (buffer [0] != '"')
	        {
		if (chars (buffer [0]))
		        {
			while (chars (buffer [i]))
			        i++;
			}

		for (j = 0; j < i; j++)
		        ptoken [j] = buffer [j];
		ptoken [i] = '\0';
		*token_length = i;
		}
	else
	        {
		do
		        {
			if ((buffer [i] == '\\') && (last_was_slash == 0))
				last_was_slash = 1;
			else
			        {
			        if ((buffer [i] == '\0') || ((buffer [i] == '"') && (last_was_slash == 0)))
				        cont = 0;
				last_was_slash = 0;
				}
			i++;
			}
		while (cont);

		for (j = 1; j < (i - 1); j++)
		        {
			if ((buffer [i] == '\\') && (last_was_slash == 0))
				last_was_slash = 1;
			else
			        {
				last_was_slash = 0;
				ptoken [k] = buffer [j];
				k++;
				}
			}
		ptoken [k] = '\0';
		*token_length = i;		
		}
	}

char token_analysis (char *ptoken)
        {
	if (strcmp (ptoken, "S") == 0)
	        return 'S';
	if (strcmp (ptoken, "s") == 0)
	        return 'S';
	if (strcmp (ptoken, "A") == 0)
	        return 'A';
	if (strcmp (ptoken, "a") == 0)
	        return 'A';
	if (strcmp (ptoken, "G") == 0)
	        return 'G';
	if (strcmp (ptoken, "g") == 0)
	        return 'G';
	if (strcmp (ptoken, "{") == 0)
	        return '{';
	if (strcmp (ptoken, "}") == 0)
	        return '}';
	if (strcmp (ptoken, "(") == 0)
	        return '(';
	if (strcmp (ptoken, ")") == 0)
	        return ')';
	if (strcmp (ptoken, "L") == 0)
	        return 'L';
	if (strcmp (ptoken, "l") == 0)
	        return 'L';
	if (strcmp (ptoken, "answer") == 0)
	        return 'a';
	if (strcmp (ptoken, "=") == 0)
	        return '=';
	return 'E';	// Unrecognized token
	}

int analys_msgtype (char *buffer, int *to_next_token_to_read, char **msgtype)
        {
	int avancer;
	int erreur;
	char ptoken [80];
	int token_length;

	erreur = 0;
	avancer = next_token (buffer);

	new_token (&(buffer [avancer]), ptoken, &token_length);
	avancer += token_length;

	switch (token_analysis (ptoken))
	        {
		case 'S':
		        {
			*msgtype = (char *) malloc (sizeof (char));
			**msgtype = 'S';
			}
		break;
		case 'A':
		        {
			*msgtype = (char *) malloc (sizeof (char));
			**msgtype = 'A';
			}
		break;
		case 'G':
		        {
			*msgtype = (char *) malloc (sizeof (char));
			**msgtype = 'G';
			}
		break;
		case 'C':
		        {
			*msgtype = (char *) malloc (sizeof (char));
			**msgtype = 'C';
			}
		break;
		case 'E':
		        {
			// Error! Token not recognized
		        erreur = -1;
			}
		break;
		default:
		        {
			// Error! Syntax not recognized
		        erreur = -2;
			}
		}

	avancer += next_token (&(buffer [avancer]));
	*to_next_token_to_read = avancer;

	return erreur;
	}

int analys_msgrange (char *buffer, int *to_next_token_to_read, char **msgrange)
        {
	int avancer;
	int erreur;
	char ptoken [80];
	int token_length;

	erreur = 0;
	avancer = next_token (buffer);

	new_token (&(buffer [avancer]), ptoken, &token_length);
	avancer += token_length;

	switch (token_analysis (ptoken))
	        {
		case '(':
		        {
			}
		break;
		case 'E':
		        {
			// Error! Token not recognized
		        erreur = -1;
			}
		break;
		default:
		        {
			// Error! Syntax not recognized
		        erreur = -2;
			}
		}

	avancer += next_token (&(buffer [avancer]));

	new_token (&(buffer [avancer]), ptoken, &token_length);
	avancer += token_length;

	switch (token_analysis (ptoken))
	        {
		case 'L':
		        {
			*msgrange = (char *) malloc (sizeof (char));
			**msgrange = 'L';
			}
		break;
		case 'E':
		        {
			// Error! Token not recognized
		        erreur = -1;
			}
		break;
		default:
		        {
			// Error! Syntax not recognized
		        erreur = -2;
			}
		}

	avancer += next_token (&(buffer [avancer]));

	new_token (&(buffer [avancer]), ptoken, &token_length);
	avancer += token_length;

	switch (token_analysis (ptoken))
	        {
		case ')':
		        {
			}
		break;
		case 'E':
		        {
			// Error! Token not recognized
		        erreur = -1;
			}
		break;
		default:
		        {
			// Error! Syntax not recognized
		        erreur = -2;
			}
		}

	avancer += next_token (&(buffer [avancer]));


	*to_next_token_to_read = avancer;

	return erreur;
	}

int analys_msgbody (char *buffer, int *to_next_token_to_read, struct maillon_message_field **msgbody)
        {
	struct maillon_message_field *msgbodycopy;
	int avancer;
	int erreur;
	char ptoken [80];
	int token_length;

	erreur = 0;
	avancer = next_token (buffer);

	new_token (&(buffer [avancer]), ptoken, &token_length);
	avancer += token_length;

	switch (token_analysis (ptoken))
	        {
		case '{':
		        {
			}
		break;
		case 'E':
		        {
			// Error! Token not recognized
		        erreur = -1;
			}
		break;
		default:
		        {
			// Error! Syntax not recognized
		        erreur = -2;
			}
		}

	avancer += next_token (&(buffer [avancer]));

	new_token (&(buffer [avancer]), ptoken, &token_length);
	avancer += token_length;

	switch (token_analysis (ptoken))
	        {
		case 'a':
		        {
			*msgbody = (struct maillon_message_field *) malloc (sizeof (struct maillon_message_field));
			(*msgbody)->field_name = (char *) malloc (7 * sizeof (char));
			(*msgbody)->field_value = NULL;
			(*msgbody)->next = NULL;
		       	strcpy ((*msgbody)->field_name, "answer\0");
			}
		break;
		case 'E':
		        {
			// Error! Token not recognized
		        return -1;
			}
		break;
		default:
		        {
			// Error! Syntax not recognized
		        return -2;
			}
		}

	msgbodycopy = *msgbody;

	avancer += next_token (&(buffer [avancer]));

	new_token (&(buffer [avancer]), ptoken, &token_length);
	avancer += token_length;

	switch (token_analysis (ptoken))
	        {
		case '=':
		        {
			}
		break;
		case 'E':
		        {
			// Error! Token not recognized
		        erreur = -1;
			}
		break;
		default:
		        {
			// Error! Syntax not recognized
		        erreur = -2;
			}
		}

	avancer += next_token (&(buffer [avancer]));

	new_token (&(buffer [avancer]), ptoken, &token_length);
	avancer += token_length;
 
	(*msgbody)->field_value = (char *) malloc ((token_length + 1) * sizeof (char));
	strcpy ((*msgbody)->field_value, ptoken);

	avancer += next_token (&(buffer [avancer]));

	while (1)
	{

	new_token (&(buffer [avancer]), ptoken, &token_length);
	avancer += token_length;

	switch (token_analysis (ptoken))
	        {
		case '}':
		        {
			avancer += next_token (&(buffer [avancer]));

			*to_next_token_to_read = avancer;
			*msgbody = msgbodycopy;

			return erreur;			
			}
		break;
		case 'a':
		        {
			(*msgbody)->next = (struct maillon_message_field *) malloc (sizeof (struct maillon_message_field));
			*msgbody = (*msgbody)->next;
			(*msgbody)->field_name = (char *) malloc (7 * sizeof (char));
			(*msgbody)->field_value = NULL;
			(*msgbody)->next = NULL;
		       	strcpy ((*msgbody)->field_name, "answer\0");
			}
		break;
		case 'E':
		        {
			// Error! Token not recognized
		        return -1;
			}
		break;
		default:
		        {
			// Error! Syntax not recognized
		        return -2;
			}
		}

	avancer += next_token (&(buffer [avancer]));

	new_token (&(buffer [avancer]), ptoken, &token_length);
	avancer += token_length;

	switch (token_analysis (ptoken))
	        {
		case '=':
		        {
			}
		break;
		case 'E':
		        {
			// Error! Token not recognized
		        return -1;
			}
		break;
		default:
		        {
			// Error! Syntax not recognized
		        return -2;
			}
		}

	avancer += next_token (&(buffer [avancer]));

	new_token (&(buffer [avancer]), ptoken, &token_length);
	avancer += token_length;

	(*msgbody)->field_value = (char *) malloc ((token_length + 1) * sizeof (char));
	strcpy ((*msgbody)->field_value, ptoken);

	avancer += next_token (&(buffer [avancer]));
	}



	*to_next_token_to_read = avancer;
	*msgbody = msgbodycopy;

	return erreur;
	}

int analys_msg (char *buffer, struct message **message)
        {
	int token_length;
	char *msgtype;
	char *msgrange;
	struct maillon_message_field *msgbody;

	msgtype = NULL;
	msgrange = NULL;
	msgbody = NULL;

	if (analys_msgtype (buffer, &token_length, &msgtype) < 0)
	        {
		printf ("Erreur dans la reconnaissance du type de message\n");
		return -1;
		}

	if (analys_msgrange (&(buffer [token_length]), &token_length, &msgrange) < 0)
	        {
		printf ("Erreur dans la reconnaissance du range du message\n");
		destroy_msgtype (&msgtype);
		return -1;
		}

	if (analys_msgbody (&(buffer [token_length]), &token_length, &msgbody) < 0)
	        {
		printf ("Erreur dans la reconnaissance du body du message\n");
		destroy_msgtype (&msgtype);
		destroy_msgrange (&msgtype);
		return -1;
		}

	printf ("\n");
	*message = (struct message *) malloc (sizeof (struct message));
	(*message)->msgtype = msgtype;
	(*message)->msgrange = msgrange;
	(*message)->msgbody = msgbody;

//	afficher_message (*message);
	return 0;
	}

int read_message (int socket, char buffer [80])
        {
	int count = 0;
	char carlu;
	int read_result;

	do
	        {
		if ((read_result = read (socket, &carlu, sizeof(char))) == 1)
		        {
			buffer [count] = carlu;
			count++;
			}
		}
	while ((carlu != '\0') && (read_result == 1));
	count--;

	if (read_result == 1)
	        return count;
	else
	        return read_result;
	}

int main (void)
	{	
	int main_socket;
	struct sockaddr_in serv_name;
	char buffer [80];
	int count;
	int count_name;
	int count_pass;
	int logout;
	char carlu;
	fd_set fdset;
	int selretval;
	struct message* message = NULL;
	int log = 0;
	char log_name [20];
	char log_pass [20];

	// termios

	int oldMin;
	tcflag_t oldCtrl;

	// termios

	printf ("pipo client v0.0b\n");
  
	if ((main_socket = socket (AF_INET, SOCK_STREAM, 0)) < 0)
		{
		perror("Fatal error: Unable to open server's main socket");
		exit (1);
		}

	serv_name.sin_family = AF_INET;
	serv_name.sin_port = htons (atoi (SERV_PORT));
	if ((inet_aton (SERV_ADDR, &serv_name.sin_addr)) == 0)
		{
		close (main_socket);
		perror( "Fatal error: Server name incorrect");
		exit (3);
		}
  
	printf ("DEBUG : ntohs (serv_name.sin_port) : ");
	printf ("%d\n", ntohs (serv_name.sin_port));
	printf ("DEBUG : ntohl (serv_name.sin_addr.s_addr) : ");
	printf ("%x\n\n", ntohl (serv_name.sin_addr.s_addr));
  
	if ((connect (main_socket, (struct sockaddr *)&serv_name, sizeof (serv_name))) < 0)
		{
		close (main_socket);
		perror( "Fatal error: Unable to connect to server");
		exit(4);
		}
  
	logout = 0;

	// termios

	termios termtype;
	tcgetattr (STDIN_FILENO, &termtype);
	oldCtrl = termtype.c_lflag;
	oldMin = termtype.c_cc [VMIN];
       	termtype.c_lflag &= !(ECHO|ICANON);
	termtype.c_cc [VMIN] = 1;
	tcsetattr (STDIN_FILENO, TCSANOW, &termtype);

	// fin termios


       	// *** LOGIN

	do
	        {
		printf ("login : ");

		count_name = 0;
		carlu = ' ';
			
		while (carlu != '\n')
		        {
			carlu = getchar ();
			printf ("%c", carlu);
			if (carlu != '\n')
			        {
		       		log_name [count_name] = carlu;
		       		count_name++;
		       		}
			}
			
		log_name [count_name] = '\0';

		if (strcmp (log_name, "Off") == 0)
		        {
			log_pass [0] = '\0';
			count_pass = 0;
			}
		else
		        {
			printf ("password : ");
			count_pass = 0;
			carlu = ' ';
			
			while (carlu != '\n')
			        {
				carlu = getchar ();
				if (carlu != '\n')
			                {
					printf ("*");
					log_pass [count_pass] = carlu;
					count_pass++;
					}
				}
			
			log_pass [count_pass] = '\0';
			printf ("\n");
		        }

		count = 19 + count_name + 15 + count_pass + 6;
		strcpy (buffer, "S (L)\n\t{\n\tlogin = \"");
		strcpy (&(buffer [19]), log_name);
		strcpy (&(buffer [19 + count_name]), "\"\n\tpassword = \"");
		strcpy (&(buffer [19 + count_name + 15]), log_pass);
		strcpy (&(buffer [19 + count_name + 15 + count_pass]), "\"\n\t}\n\0");

		write (main_socket, buffer, count);

		if ((count = read_message (main_socket, buffer)) < 0)
		        {
			close (main_socket);
			perror ("Fatal error: Unable to read message from server");
			exit(5);
			}

		if (analys_msg (buffer, &message) < 0)
		        {
			printf (" *** Erreur d'analyse de l'ack de log! ***\n\n");
			}
		else
		        {
			if ((message != NULL)
			    && (message->msgtype != NULL)
			    && (message->msgrange != NULL)
			    && (message->msgbody != NULL)
			    && (*(message->msgtype) == 'A')
			    && (*(message->msgrange) == 'L')
			    && (message->msgbody->field_name != NULL)
			    && (message->msgbody->field_value != NULL)
			    && (message->msgbody->next == NULL)
			    && (strcmp (message->msgbody->field_name, "answer") == 0)
			    )
			        {
				if (strcmp (message->msgbody->field_value, "ok")  == 0)
				        {
				        log = 1;
					}
				else if (strcmp (message->msgbody->field_value, "wrong password")  == 0)
				        {
					printf ("Wrong password. Please retry.\n\n");
					}
				else if (strcmp (message->msgbody->field_value, "bad user")  == 0)
				        {
					printf ("User doesn't exist. Please retry.\n\n");
					}
				else if (strcmp (message->msgbody->field_value, "stop execution")  == 0)
				        {
					printf ("Server deconnection. Bye Bye.\n\n");
					close (main_socket);
					exit (0);
					}
				else if (strcmp (message->msgbody->field_value, "protocol error")  == 0)
				        {
					printf ("Server couldn't understand. Bye Bye.\n\n");
					close (main_socket);
					exit (0);
					}
				else
				        {
					printf ("Ouch! Couldn't understand server answer.\n\n");
					}
				}
			}

	        destroy_message (&message);
		}
	while (!log);

	// *** END OF LOGIN

  	printf ("Connected to BBS.\nWelcome %s.\n\n", log_name);
  


	while (!logout)
		{
		memset (buffer, 0, sizeof (buffer));
		count = 0;

		FD_ZERO (&fdset);
		
		FD_SET (0, &fdset);
		FD_SET (main_socket, &fdset);

		#ifdef HP_UX
		selretval = select ((main_socket + 1), (int*) &fdset, NULL, NULL, 0);
		#else
		selretval = select ((main_socket + 1), &fdset, NULL, NULL, 0);
		#endif

		if (selretval < 0)
			{
			close (main_socket);
			perror ("Fatal error: Select error");
			exit (5);
			}

		if FD_ISSET (main_socket, &fdset)
			{
			printf ("Incoming message!\n");

			if ((count = read_message (main_socket, buffer)) < 0)
				{
				close (main_socket);
				perror ("Fatal error: Unable to read message from server");
				exit (5);
				}

			printf ("Network client received the message :\n\"%s\"\nfrom server\n\n", buffer);
			if (analys_msg (buffer + 1 /* pour virer le '>' dans le message. Temporaire. */, &message) < 0)
			        {
				printf (" *** Erreur d'analyse du message! ***\n\n");
				}
		       	destroy_message (&message);
			}

		else if FD_ISSET (0, &fdset)
			{
			printf ("Sending X-Message\n");
			carlu = ' ';
			count = 0;
			
			while (carlu != '\n')
				{
				carlu = getchar ();
				printf ("%c", carlu);
				if (carlu != '\n')
					{
					buffer [count] = carlu;
					count++;
					}
				}
			
			buffer [count] = '\0';
			count++;
			
			write (main_socket, buffer, count);
			
			if (buffer [0] == '\0')
				{
				printf ("Sending data to close connection\n");
				logout = 1;
				}
			else
				printf ("Network client send the message :\n\"%s\"\nto server\n\n", buffer);
			}
			
		else
			{
			close (main_socket);
			perror ("Fatal error: Unable to locate the source of select return");
			exit (6);
			}
		}

	//termios

	termios termtype2;
	tcgetattr (STDIN_FILENO, &termtype2);
	termtype.c_lflag = oldCtrl;
	termtype.c_cc [VMIN] = oldMin;
	tcsetattr (STDIN_FILENO, TCSANOW, &termtype2);

	//termios

	memset (buffer, 0, sizeof (buffer));
	count = 0;

	if ((count = read_message (main_socket, buffer)) < 0)
		{
		close (main_socket);
		perror ("Fatal error: Unable to read deconnection message from server");
		exit (5);
		}

	printf ("Network client received the deconnection message :\n\"%s\"\nfrom server (byebye)\n", buffer);
	printf ("Byebye\n");
 
	if ((close (main_socket)) < 0)
		{
		perror( "Fatal error: Unable to close server's main socket");
		exit (2);
		}

	printf ("Client connection correctly closed\n\n");
	return 0;
	}


void terminate (void)
	{
	printf ("Ze termine\n");
	exit (0);
	}
