#ifndef __CLIENTCOMM__
#define __CLIENTCOMM__

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <termios.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/wait.h>
#include <sys/time.h>
#include <arpa/inet.h>

#define SERV_PORT "2000"
#define SERV_ADDR "139.124.29.1"


class Struct_Message
        {
        protected:
	struct maillon_message_field;

	struct maillon_message_field
                {
		char *field_name;
		char *field_value;
		struct maillon_message_field *next;
                };

	struct message
                {
		char *msgtype;
		char *msgrange;
		struct maillon_message_field *msgbody;
                };

        protected:
	struct message *mesg;

        public:
	Struct_Message (void);
	~Struct_Message (void);
	void destroy_message (void);
	void new_message (char, char);
	int add_field (char *, char *);
	int add_field (char *, char);
	int add_field (char *, unsigned short);
	char *get_type (void);
	char *get_range (void);
	char *get_field_value (char *);

        protected:
	void afficher_msgbody (struct maillon_message_field *);
	void afficher_message (void);
	void destroy_msgtype (char **);
	void destroy_msgrange (char **);
	void destroy_string (char **);
	void destroy_maillon (struct maillon_message_field **);
	void destroy_msgbody (struct maillon_message_field **);
	struct message *message_malloc (void);
	struct maillon_message_field *field_malloc (void);
	void define_message (void);
	int define_type (char);
	int define_range (char);
	int define_field (void);
	int define_field_name (char *);
	int define_field_value (char *);	
	int us_size (unsigned short);
	void us_to_a (char *, unsigned short);
        };


class Struct_Buffer
        {
        protected:
	struct maillon_buffer;

	struct maillon_buffer
	        {
		char car;
		struct maillon_buffer *next;
	        };

        protected:
	struct maillon_buffer *buff;
	struct maillon_buffer *last;

        public:
	Struct_Buffer (void);
	~Struct_Buffer (void);
	void destroy_buffer (void);
	void add_char_to_buffer (char);
	void add_string_to_buffer (char *);
	int read_buffer_from_input (class Socket *);
	int buff_count (void);
	void buff_to_buffer (char *);

        protected:
	void afficher_buffer (void);
	struct maillon_buffer *maillon_malloc (void);
        };


class Message : public Struct_Message, public Struct_Buffer
        {
        public:
	Message (void);
	~Message (void);
	int convert_buff_to_mesg (void);
	int convert_mesg_to_buff (void);

        private:
	int next_token (char *);
	int chars (char);
	void new_token (char *, char *, int *);
	char token_analysis (char *);
	int analys_msgtype (char *, int *, char **);
	int analys_msgrange (char *, int *, char **);
	int analys_msgbody (char *, int *, struct maillon_message_field **);
	void put_string (struct maillon_buffer **, char *);
       	int analys_msg (char *);
        };


class Message_Input : public Message
        {
        public:
	Message_Input (void);
	~Message_Input (void);
	int read_from_input (class Socket *);

        private:

        protected:
        };


class Message_Output : public Message
        {
        public:
	Message_Output (void);
	~Message_Output (void);
	int write_to_output (class Socket *);

        private:

        protected:
        };


class Socket
        {
        private:
	int main_socket;
	struct sockaddr_in serv_name;

        public:
	Socket (void);
	~Socket (void);
	void init_connect (void);
	void close_connect (void);
	int get_socket_fd (void);
	int read_char (char *);
	int write_string (char *);
	int get_message_from_socket (class General_Message **);

        private:
	unsigned short a_to_us (char *);
        };


class General_Message
        {
        public:
	General_Message (void);
	virtual ~General_Message (void);
	virtual char *what (void);
        };


class X_Message : public General_Message
        {
        private:
	char type;
	unsigned short user;
	char *body;

        public:
	X_Message (void);
	virtual ~X_Message (void);
	virtual char *what (void);
	void define_type (char);
	void define_user (unsigned short);
	void define_body (char *);
	char get_type (void);
	unsigned short get_user (void);
	int get_body_size (void);
	void get_body (char *);
	int send (class Socket *);

        private:
	void destroy_body (void);

        protected:
        };


class Login_Attempt : public General_Message
        {
        private:
	char *login;
	char *password;

        public:
	Login_Attempt (void);
	virtual ~Login_Attempt (void);
	void define_login (char *);
	void define_password (char *);
	int send (class Socket *);

        private:
	void destroy_login (void);
	void destroy_password (void);

        protected:
        };


class Login_Answer : public General_Message
        {
        private:
	char *answer;

        public:
	Login_Answer (void);
	virtual ~Login_Answer (void);
	virtual char *what (void);
	void define_answer (char *);
	int get_answer_size (void);
	void get_answer (char *);

        private:
	void destroy_answer (void);

        protected:
        };



#endif
