/*******************************/
/* SYNOPSIS : runserver [port] */
/*******************************/

#include "Server.H"

G_Path G (0);	// Process ne s'arretant jamais
Con_Wholist W;	// Te permet mater mem partagee
C_Pipe P;


void terminate (void)
	{
	printf ("Ze termine\n");
	exit (0);
	}

void server_child_termination (int sign)
	{
	if (sign == SIGCHLD)
		{
		signal (SIGCHLD, server_child_termination);	// relaunch the signal
		wait (NULL);
		}
	}

void server::server_listen_create (void)
        {
	/***********************************************/
	/* Creation du socket d'ecoute 'listen_socket' */
	/* Pour la connection du serveur sur le server */
	/***********************************************/

	if ((listen_socket = socket (AF_INET, SOCK_STREAM, 0)) < 0)
		{
		perror ("Fatal error: Unable to open server's listen socket");
		exit (SLSOCK_CREATE);
		}
	}

void server::server_name_identify (char *serv_port)
        {
	/***********************************************/
	/* On remplit la structure bbs_server avec les */
	/* caracteristiques du listen_socket:	       */
	/* 1) sin_family : type de connection	       */
	/* 2) sin_port : port			       */
	/* 3) sin_addr : adresse internet du bbs (IP)  */
	/***********************************************/
  
	server_name.sin_family = AF_INET;
	server_name.sin_port = htons (atoi (serv_port));
	server_name.sin_addr.s_addr = INADDR_ANY; // -> getsockname
	}

void server::server_bind (void)
        {
	/**********************************************************/
	/* Tentative de bind du listen_socket avec le server_name */
	/**********************************************************/

	if ((bind (listen_socket, (struct sockaddr *)&server_name, sizeof (server_name))) < 0)
		{
		close (listen_socket);
		perror ("Fatal error: Unable to bind server's listen socket");
		exit (SLSOCK_BIND);
		}
	}

void server::server_getsockname (void)
        {
	/*********************************************************************/
	/* Ici on obtient le nom du serveur et on le stocke dans server_name */
	/*********************************************************************/

	server_len = sizeof (server_name);

	if ((getsockname (listen_socket, (struct sockaddr *)&server_name, &server_len)) < 0)
		{
		close (listen_socket);
		perror ("Fatal error: Unable to get server's listen socket name");
		exit (SLSOCK_GETSOCKNAME);
		}
	}

void server::server_listen_mode (void)
        {
	/********************************************/
	/* On passe le listen_socket en mode listen */
	/********************************************/
  
	if ((listen (listen_socket, 5)) < 0)
		{
		close (listen_socket);
		perror ("Fatal error: Unable to listen to server's listen socket");
		exit (SLSOCK_LISTEN);
		}
	}

void server::server_comm_create (void)
        {
	/*********************************/
	/* Create the communication pipe */
	/* to administrate the server	 */
	/*********************************/

	if ((pipe (comm_pipe)) < 0)
		{
		close (listen_socket);
		perror ("Fatal error: Unable to create server's communication pipe");
		exit (SCPIPE_CREATE);
		}
	}

server::server (char *serv_port = SERV_PORT)
	{	
	printf ("pipo server v0.0b\n");

	server_listen_create ();
	server_name_identify (serv_port);
	server_bind ();
	server_getsockname ();

	printf ("DEBUG : ntohs (server_name.sin_port) : ");
	printf ("%d\n", ntohs (server_name.sin_port));
	printf ("DEBUG : ntohl (server_name.sin_addr.s_addr) : ");
	printf ("%x\n", ntohl (server_name.sin_addr.s_addr));

	server_listen_mode ();
	server_comm_create ();
	}

char server::verify_login (char *login_name, char *login_pass)
        {
	if (strcmp (login_name, "Off") == 0)
	        return 'S';
	if (strcmp (login_name, "Gronk") == 0)
	        return 'L';
	if (strcmp (login_pass, "pipo") == 0)
	        return 'O';
	return 'P';
	}

void server::signal_update (const int signal)
        {
	/*********************************************************/
	/* On prevoit la reception de signaux, comme  SIGCHLD	 */
	/* prevenant du deces des child process qui se terminent */
	/*********************************************************/

	if (signal == SIGCHLD)
	        signal (SIGCHLD, server_child_termination);
	}

void server::init_server_set (int *maxfdinset, fd_set *fd_serv_set)
        {
	/*********************************/
	/* Initialize the set with no fd */
	/*********************************/

	zero_fd (maxfdinset, fd_serv_set);

	/****************************************************/
	/* Add the comm server's pipe and socket to the set */
	/****************************************************/
					
	set_fd (maxfdinset, listen_socket, fd_serv_set);
	set_fd (maxfdinset, comm_pipe [0], fd_serv_set);
	}

void server::server_select (int maxfdinset, int *nbselret, fd_set *fd_serv_set)
        {
	/***********************************************/
	/* Waiting for an incoming message on the pipe */
	/* or on the socket with select		       */
	/* N.B: HP_UX needs her select having 2nd arg  */
	/* to be int* type			       */
	/***********************************************/

        #ifdef HP_UX
	*nbselret = select (maxfdinset, (int*) fd_serv_set, NULL, NULL, 0);
        #else
	*nbselret = select (maxfdinset, fd_serv_set, NULL, NULL, 0);
        #endif
	}

void server::test_nbselret (int nbselret, int *relaunch_accept)
        {
	if (nbselret <= 0)
	        {
		if (errno != EINTR)	// La c'est pas normal
		        {
			close (listen_socket);
			perror ("Fatal error: Unable to accept a connection from client");
			exit (PARENT_SELECT);
			}
		else			// La c'est normal: un fils est mort
		        {
			printf ("Connection status: correctly closed\n");
			*relaunch_accept = 1;
			}
		}
	}

void server::server_wait (int maxfdinset, fd_set *fd_serv_set)
        {
	int relaunch_accept = 0;
	int nbselret;

	do
	        {
		server_select (maxfdinset, &nbselret, fd_serv_set);
		relaunch_accept = 0;
		test_nbselret (nbselret, &relaunch_accept);
		}
	while (relaunch_accept);
	}

void server::server_admin_message (void)
        {
	printf ("Incoming child message!!!\n");
	}

void server::server_accept_test (int *relaunch_accept)
        {
	if ((client_socket = accept (listen_socket, (struct sockaddr *)&client_name, &client_len)) < 0)
	        {
		if (errno != EINTR)	// La c'est pas normal
		        {
			close (listen_socket);
			perror ("Fatal error: Unable to accept a connection from client");
			exit (SLSOCK_ACCEPT);
			}
		else			// La c'est normal: un fils est mort
		        {
			printf ("Connection status: correctly closed\n");
			*relaunch_accept = 1;
			}
		}
	}

void server::server_client_accept (void)
        {
	int relaunch_accept = 0;

	/**************************************/
	/* On attend une connection du client */
	/* a l'aide de la primitive accept    */
	/**************************************/

	do
	        {
		relaunch_accept = 0;
		server_accept_test (&relaunch_accept);
		}
	while (relaunch_accept);
	} 

void server::server_child_connect (int *logout, int *pid)
        {
	client_len = sizeof (client_name);

	server_client_accept ();




			/********************************************/
			/* Il est tant de forker le serveur :	    */
			/* Le pere reste celui qui fait les accepts */
			/* Le fils gere la nouvelle connection	    */
			/********************************************/

			if ((*pid = fork ()) < 0)
				{
				close (client_socket);
				close (listen_socket);
				perror ("Fatal error: Fork Failure\n");
				exit (PARENT_FORK);
				}
				
			if (*pid != 0)	// Parent process
				{
				/***********************************************/
				/* On ferme le socket qu'on vient de connecter */
				/* Et on reprend l'attente d'accept	       */
				/***********************************************/

				close (client_socket);

				}	// End of parent

			else		// Child process
				{
				int result_log;
				char *login_name;
				char *login_pass;

				login_name = (char *) malloc (20 * sizeof (char));
				login_pass = (char *) malloc (20 * sizeof (char));

				/***********************************************/
				/* On ferme le socket listen du serveur car    */
				/* l'accept est gere par le pere	       */
				/* We also close the comm_pipe [0] which reads */
				/* commands and is only used by parent	       */
				/***********************************************/

				close (listen_socket);
				close (comm_pipe [0]);

				/************************************************/
				/* Creation of the conection to the pipe server */
				/* Mon boulot :					*/
				/************************************************/

				Socket ToClient (client_socket);

				do
				        {
					login_name [0] = '\0';
					login_pass [0] = '\0';
					result_log = ToClient.GetLogin (&login_name, &login_pass);

					if (result_log < 0)
					        {
						ToClient.SendAckLog ("protocol error");
						printf ("Unable to understand login message.\n");
						close (listen_socket);
						perror ("Client unable to give log and pass");
						exit (CL_CANNOT_LOG);
						}
					else
					        {
						switch (verify_login (login_name, login_pass))
						        {
							case 'O':
							        {
								ToClient.SendAckLog ("ok");
								printf ("Accept log %s.\n", login_name);
								}
							break;
							case 'S':
							        {
								ToClient.SendAckLog ("stop execution");
								printf ("Closing connection (%s user).\n", login_name);
								close (listen_socket);
								exit (CL_ASK_LOGOUT);
								}
							break;
							case 'P':
							        {
								ToClient.SendAckLog ("wrong password");
								printf ("Wrong pass for %s\n", login_name);
								result_log = -1;
								}
							break;
							case 'L':
							        {
								ToClient.SendAckLog ("bad user");
								printf ("User %s doesn't exist\n", login_name);
								result_log = -1;
								}
							break;
						        }
						}
					}
				while (result_log < 0);

				C_User_C ToServer (login_name);	// login_name contient le nom du client

				/*****************************************************/
				/* On indique au socket l'adresse du pipe du serveur */
				/*****************************************************/

				ToClient.Set (&ToServer);
			
				/****************************************************/
				/* Boucle principale de reception/envoi de messages */
				/****************************************************/

				while (!(*logout))
					{ 
					try
						{
						fd_set fdset;	      // File descriptor set for select
						int nbpret;	      // Return value of select
						int maxfd;	      // Max value +1 fd within a set
		  
						zero_fd (&maxfd, &fdset);	// init set with no fd

						/***********************************************/
						/* Add the client's pipe and socket to the set */
						/***********************************************/
					
						set_fd (&maxfd, ToServer.nompipe, &fdset);
						set_fd (&maxfd, client_socket, &fdset);
					
						/***********************************************/
						/* Waiting for an incoming message on the pipe */
						/* or on the socket with select		       */
						/* N.B: HP_UX needs her select having 2nd arg  */
						/* to be int* type			       */
						/***********************************************/
					
						#ifdef HP_UX
						nbpret = select (maxfd, (int*) &fdset, NULL, NULL, 0);
						#else
						nbpret = select (maxfd, &fdset, NULL, NULL, 0);
						#endif
				
						if (nbpret <= 0)
							{
							perror ("Fatal error: Select failure\n");
							exit (CHILD_SELECT);
							}

						if FD_ISSET (client_socket, &fdset)	// Receiving a message from distant client
							{
							W.IDLE ();			// User is alive and not idle
							ToClient.Gere ();
							}
						
						if FD_ISSET (ToServer.nompipe, &fdset)	// Receiving a message from bbs
							{
							unsigned char ch;
						
							ToServer.Lit (ch);
							switch (ch)
								{
								case 'X':
									{
									printf ("Receiving X from server\n");
									Un_X a;
									ToServer.Get_X (a);
									printf ("Sending x to client\n");
									ToClient.SendX (a.buf, a.ki, a.type);
									}
								break;
								case ' ':
									{
									printf ("Le serveur vient de m'envoyer un post :)\n");
									}
								break;
								case 'N':
									{
									printf ("Le serveur me demande de relire les infos\n");
									ToServer.Lit_Sauve (k_Read);
									printf ("J'envoie un pipo X a AA\n");
									ToClient.SendX ("AA, t un pipo\n", 0, '.');
									}
								break;
								case 'a': case 'A':
									{
									printf ("Receiving an acknowledgment of X\n");
									Un_Ack a;
									ToServer.Get_Ack (ch == 'a', a); 
									printf ("AA, tu feras ce k'il faut\n");			     
									}
	  							break;
								case 'R': 
									{
									printf ("Le serveur me signale que je change de roume\n");
									Un_Roume a;
									ToServer.Get_Room (a); 
									printf ("AA, au turf\n");
									}
								break;
								case 'S': case 'K':
									{
									printf ("Receiving a Connect/Deconnect signal from server\n");
									// Hello_Baille a (ch =='S');
									if (ch == 'S')
										{
										printf ("Le serveur dit hello\n");
										W.Set (&ToServer);
										}
									else 
										{
										printf ("le serveur dit baille\n");
										throw Deconnection ();
										}
									}
								break;			   
								case 'h': case 'b':
									{
									printf ("Le serveur signale kkun ki se loge ou deloge\n");
									Un_log a;
									ToServer.Get_Log (ch == 'h', a); 
									printf ("AA, o turf !\n");
									}
								break;	     
								case 'k': 
									{
									printf ("Le serveur m'envoie la liste des roumes\n");
									RListe a;
									ToServer.Get_RListe (a); 
									printf ("AA, o turf !\n");
									}
								break;
								case 'T': 
									{
									printf ("Le serveur m'envoie une info sur le tag :)\n");
									Un_Tag a;
									ToServer.Get_Tag (a);
									printf ("AA, o turf !\n");
									}
								break;
								default:
									printf ("error protocole: %c(%d)\n",ch,ch);
								}
							}		  
						}	// Fin de Try

					catch (Deconnection)
						{
						printf ("Server asking for deconnection\n");
						*logout = 1;
						}

					// Gere les autres catch
					// PipeProbs, PlusPersonne, ....

					}	// Fin while (!logout) du fils
				free (login_name);
				free (login_pass);
				}		// Fin du process du fils
	}

void server::server_incoming_connection (fd_set *fd_serv_set, int *logout, int *pid)
        {
	if FD_ISSET (comm_pipe [0], fd_serv_set) // Receiving an administration message from child
		server_admin_message ();
	if FD_ISSET (listen_socket, fd_serv_set) // Receiving a login attempt from distant client
	        server_child_connect (logout, pid);
	}

int server::end_of_run (int pid)
        {
	/**********************************************/
	/* La tache de ce serveur est finie.	      */
	/* Et la poussiere retourne a la poussiere... */
	/* Si c'est un fils, on exit pour eviter      */
	/* qu'il poursuive son execution.	      */
	/**********************************************/
  
	if (pid == 0) // C'est le fils :
		{
		printf ("Child server correctly logged out\n");
		exit (0); // On exite le programme pour stopper l'execution du process.
		}
	else	      // C'est le pere :
		{
		printf ("Server's run correctly terminated\n");
		return 0; // On retourne 0 et on continue l'execution.
		}
	}

int server::run (void)
	{
	int pid = 0;		// Contient le pid du dernier fils forke
	int logout = 0;		// Condition d'arret du serveur
	int endval;

	signal_update (SIGCHLD);

	while (!logout)
		{
		fd_set fd_serv_set;		// File descriptor set for select
		int maxfdinset;			// Max value +1 fd within a set
					
		init_server_set (&maxfdinset, &fd_serv_set);
		server_wait (maxfdinset, &fd_serv_set);
		server_incoming_connection (&fd_serv_set, &logout, &pid);
		}

	endval = end_of_run (pid);

	return endval;
	}

void server::zero_fd (int *max, fd_set *fdset)
	{
	FD_ZERO (fdset);
	*max = 0;
	}

void server::set_fd (int *max, int fd, fd_set *fdset)
	{
	FD_SET (fd, fdset);
	if ((fd + 1) > *max)
		*max = fd + 1;
	}

int main (void)
	{
	server serv;
	return serv.run ();
	}
