/* ***************************************************************** *
 * Atlantis BBS - Rooms Lister CGI                                   *
 * ----------------------------------------------------------------- *
 * Version . . . . . . . . . . . . . . . . . . . . . . . . . . 1.0.0 *
 * ================================================================= *
 * Creation date . . . . . . . . . . . . . . . . . . Wed 25 Nov 1998 *
 * Modification date . . . . . . . . . . . . . . . . Thu 03 Nov 1998 *
 * ================================================================= *
 * Copyright (C) 1997-1998 Free Software Foundation, Inc.            *
 *                                                                   *
 * This program  is  free software;  you can  redistribute it and/or *
 * modify it  under the terms  of the GNU  General Public License as *
 * published by  the Free Software Foundation;  either version 2, or *
 * (at your option) any later version.                               *
 *                                                                   *
 * This program  is distributed in the hope  that it will be useful, *
 * but WITHOUT  ANY WARRANTY;  without even  the implied warranty of *
 * MERCHANTABILITY  or  FITNESS  FOR A  PARTICULAR PURPOSE.  See the *
 * GNU General Public License for more details.                      *
 *                                                                   *
 * You should have received a copy of the GNU General Public License *
 * along  with  this  program;  if not,  write to  the Free Software *
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.         *
 *                                                                   *
 * ================================================================= *
 * Authors                                                           *
 *   Sebastien APERGHIS-TRAMONI <saper@esil.univ-mrs.fr>             *
 *                                                                   *
 * If you find any bug, or if you do any improvment, please send us  *
 * a mail (possibly with diff and/or correction).                    *
 *                                                                   *
 * ***************************************************************** */


/* ***************************************************************** *
 * standart includes                                                 *
 * ***************************************************************** */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>


/* ***************************************************************** *
 * Pipo-BBS includes                                                 *
 * ***************************************************************** */
#include "consts.H"
#include "MAX.H"


/* ***************************************************************** *
 * CGI includes                                                      *
 * ***************************************************************** */
#include "CGI_colors.h"
#include "CGI_parser.h"
#include "CGI_HTML.h"
#include "CGI_room.h"
#include "log_error.h"


/* ***************************************************************** *
 * global variables                                                  *
 * ***************************************************************** */
static char program[]           = "Rooms Lister CGI";
static char version[]           = "1.0.0a4";

static char Language[]          = "lang";
static char Default_Language[]  = "en";
static char Room_Name[]         = "id";
static int  Nb_Columns          = 3;

static char*lang                = NULL;



/* ***************************************************************** *
 * main()                                                            *
 *                                                                   *
 * ***************************************************************** */
int main(void) {
    HASH *  data = NULL;
    char    name[LG_NOM_ROOM+1];
    int     flags;
    int     column = 0;
    int     i = 0;
    
    open_log_file();
    setbuf(stdout, NULL);  // Unbuffer stdout.
    setbuf(stderr, NULL);  // Unbuffer stderr.
    
    data = expect(5);
    parse(data);
    
    lang = get_value(data, Language);
    
    if(lang == NULL) {
        lang = Default_Language;
    }
    
    
    // Sending HTTP Header
    Standard_HTTP_Header();
    
    // Sending HTML document
    Standard_HTML_Top("Islands");
    
    printf("<P Align=Justify>\n&nbsp;&nbsp;"
           "Here are the islands currently available on Atlantis "
           "BBS. You can click the name of a particular island to "
           "get more information about it.</P>\n\n"
           "<HR>\n"
           "<TABLE Border=0 CellSpacing=0 Align=Center>\n"
    );
    
    for(i=0; i<MAX_ROOM; i++) {
        
        if(get_room_info(i, name, &flags) && strlen(name) > 0) {
            
            if(column == 0) {
               // First column
               printf("  <TR>\n");
            }
            
            printf("    <TD Align=Right>%d. </TD> "
                   "<TD><A HREF=\"/cgi-bin/RoomInfo?%s=%s&%s=%d\">%s</A>"
                   "&nbsp;&nbsp;&nbsp;</TD>\n", 
                   i, Language, lang, Room_Name, i, name
            );
            
            if(column == Nb_Columns - 1) {
                // Last column
                puts("  </TR>\n");
            }
            
            column = (column + 1) % Nb_Columns;
        }
    }
    
    printf("</TABLE>\n");
    
    Standard_HTML_Bottom(program, version);
    
    return 0;
}


/* ***************************************************************** *
 * terminate()                                                       *
 *                                                                   *
 * ***************************************************************** */
void terminate(void) {
  fprintf(stderr, "%s: terminate()\n", program);
  exit(1);
}


/*
const unsigned short R_subject= 1<< 0; // Cette room a des sujects
const unsigned short R_anonym = 1<< 1; // On peut poster anonymous
const unsigned short R_invite = 1<< 2; // il faut etre invite pour pouvoir poster
const unsigned short R_invisi = 1<< 3; // il faut etre invite pour voir room
const unsigned short R_1_only = 1<< 4; // Only one person can post in a time
// 1 << 5: nothing
const unsigned short R_male   = 1<< 6; // C une room mec   ( ? ) 
const unsigned short R_female = 1<< 7; // C une room nana  ( ? ) 
const unsigned short R_scram  = 1<< 8; // post en scramble ( ? )
const unsigned short R_inv    = 1<< 9; // post en inverse  ( ? )
*/
